#!BPY
# S.C.E.
# a Blender City creation script
#	This script allows you to create cities that
#	 look fairly realistic from a closer and closer
#	distance, as the script evolves.
#	it relies on a core + a plugin system, that
#	allows to add new procedural building codes,
#	aka 'building libraries'. mix them together
#	in the same city.
# 	documentation, logs, roadmap, download :
#	 http://jerome.le.chat.free.fr

# ###################
# Alt-p to execute  #
# ###################

# author: Jerome Mahieux (littleneo) paris, france
# first script author : Arnaud Couturier (piiichan) New Caledonia (Pacific island)
# contributors :
# Kurt Whicher (another_noob) : v0.2 - naming convention, detailed building option in the GUI
# Zsolt Fartek (b3d.hu) : hungarian translation

# creation date: November 2008 (Blender 2.48a)
# last changes : 14 mai 2009
sceVersion="0.39.9b"

# license: free for any use

import Blender #time,fpformat,string,Blender,math
from Blender import Mesh,Object,Draw,BGL,Scene,Image,Camera,Material,Texture,Window,sys,Mathutils
import bpy
from random import randint,random,uniform
from Blender.BGL import *
from Blender.Mathutils import *
import os,traceback,string,shutil,sys as Psys
from math import *

syco=False
# psyco tests
if syco :
	try :
		import psyco
		psyco.log()
		psyco.profile(0.2)
	except : pass


path=sys.expandpath("//")
failed=""

# common functions
# #################

# list a directory
# second argument can be :
# "dir" or 0 to list subdirs only,
# "file" or 1 to list files only,
# or ".extension" to list only files with that extension.

def fromLib(libname) :
	global BLIB
	for id,lib in enumerate(BLIB) :
		if lib[0][0]==libname :
			return id

def retrievefromgui(libname,string) :
	global BLIB
	for lib in BLIB :
		if lib[0][0]==libname :
			for i,list in enumerate(lib[1]) :
				if list[-1]==string :
					return list[0].val
					
def findType(var) :
	try :
		try : 
			test=var.index(".")
			var=float(var)
		except :
			var=int(var)
	except :
		if var.lower()=="true" :var=bool(1)
		elif var.lower()=="false" :var=bool(0)
		else : var=str(var)
	return var
		
def libReset() :
	global libGui,libName,libAuthor,libComment,libVersion,libDate
	libGui=False
	libName=""
	libAuthor=""
	libComment=[]
	libVersion=""
	libDate=""

def dirContent(path,what="all") :
	fileList = [os.path.normcase(f) for f in os.listdir(path)]
	if what=="all" : return fileList
	if what=="file" or str(what)=="1" : what=1
	elif what=="dir" or str(what)=="0" : what=0
	elif what[0]=="." :
		fileList = [f for f in fileList if os.path.isfile(os.path.join(path, f))==1 and f[-len(what):]==what ]
		return fileList
	fileList = [f for f in fileList if os.path.isfile(os.path.join(path, f))==what]
	return fileList

def savePresets(file) :
	global path,CATS,BLIB,MSET,presets,none_text
	# filter params :
	if TLIM[0].val==none_text : TLIM[0].val=""
	if TLIMB[0].val==none_text : TLIMB[0].val=""

	guiValues = open(path+"/bin/"+"preset.out", "r")
	if file.find("/")<>-1 or file.find("\\")<>-1 : # save as (path and perhaps ext included) 
		if file[-4:]<>".set" :file=file+".set"
		setname=file[max(file.rfind("/"),file.rfind("\\"))+1:-4]
		presets.append(setname)
		outFile = open(file, "w")
		MSET[0].val=len(presets)-1
	else : 
		outFile = open(path+"/presets/"+file+".set", "w") # save (file name from a list)
		setname=file
	dprint("save set '%s'",setname)
	# library stuff : category and variables
	CATSAVE=[]
	LIBSAVE=[[],[]]
	# buildingLibCategory=CATSAVE :
	for i,c in enumerate(CATS):
		if c[1][0][0].val :
			if i==0 : CATSAVE.append([c[0]])
			# save filters config
			else : CATSAVE.append([c[0],c[1][2][0].val,c[1][3][0].val,c[1][4][0].val,c[1][5][0].val,c[1][6][0].val])
			catlibs=[]
			for j,l in enumerate(c[-1]) :
				if l[1][0].val :
					catlibs.append([l[0],l[2][0].val])
					# prepare libraries arguments
					try : test=LIBSAVE[0].index(l[0])
					except : 
						for lib in BLIB :
							#print lib[0][-1]
							if lib[0][0]==l[0] and lib[0][-1]:
								LIBSAVE[0].append(l[0])
								libargs=[]
								for i,list in enumerate(lib[1]) :exec("libargs.append('"+list[-1]+"="+str(list[0].val)+"')")
								LIBSAVE[1].append(libargs)
				
			CATSAVE[-1].append(catlibs)
	# save global generic parameters
	for line in guiValues.readlines():
		line=chomp(line)
		arg = line.split("=")
		if arg[0]=="" :
			outFile.write('\n')
		elif arg[0][0]=="#" :
			outFile.write(line+'\n')
		else :
			exec("val="+arg[1])
			if type(val)==type(str()) :quote="'"
			else : quote=""
			outFile.write(arg[0]+"="+quote+str(val)+quote+"\n")
	#save libraries arguments
	for i,l in enumerate(LIBSAVE[0]):
		outFile.write("<"+l+">\n")
		for args in LIBSAVE[1][i] :
			outFile.write(args+"\n")
	
	outFile.close()
	guiValues.close()
	
def loadImage(file,dest="") :
	global path,TLIM,TLIMB,none_text,imageFor
	if file.find(path[0:-1])==0 : 
		file= "//"+file[len(path[0:-1]):]
		dprint("convert to local path : %s",file,3)
	filename=file[file.rfind(sys.sep)+1:]
	file=file.replace("\\", "/")
	dprint("load image file %s...",filename,level=5)
	try :
		image=Image.Load(file)
		try : 
			image.getSize()
			dprint("done.",level=5)
		except :
			dprint("read error.",level=5)
			file=none_text
	except :
		dprint("not found.",level=5)
		file=none_text
	if dest=="" : dest=imageFor
	if dest=="building" : TLIMB[0].val=file
	else : TLIM[0].val=file
	
	
def erasePreset(name) :
	global path
	dprint("delete preset "+name+".")
	os.remove(path+"/presets/"+name+".set")

	
def include(name,type="py",subPath="",section="",cat="",funcexecname="") :
	global path,BLIB,CATS
	if subPath=="":
		if type=="py":subPath="/bin/"
		elif type=="set":subPath="/presets/"
		elif type=="lang":subPath="/locale/"
	file = open(path+subPath+name+"."+type, "r")
	if type=="py" :exec file in globals()
	# include lib
	elif type=="lib" :
		thisLib=fromLib(subPath.split(sys.sep)[-2])
		if section=="" :section="function"
		funcnum=-1
		nblines=0
		headerlib=False
		varlib=False
		guilib=False
		funclib=False
		if section=="function" : library=[[],[]]
		else : library=[]
		#print "update %s (section %s)"%(thisLib,section)
		for line in file.readlines():
			nblines +=1
			line=chomp(line)
			read=False
			if line=="<header>" : headerlib=True
			elif line=="</header>" : headerlib=False
			elif line=="<var>" : varlib=True
			elif line=="</var>" : varlib=False
			elif line=="<gui>" : guilib=True
			elif line=="</gui>" : guilib=False
			elif line[0:9]=="<function" :
				funclib=True
				if section=="function" :
					if line[9]==">" : funcname="generic"
					else : funcname=line[10:line.index(">")]
					funcnum +=1
					library[0].append(funcname)
					library[1].append("#\n")
					# dummy lines to know  right line number when a bug occurs
					for blank in range(nblines-3) : library[1][funcnum]=str(library[1][funcnum])+"#\n"
			elif line[0:10]=="</function" : funclib=False
			else : read=True
			
			if read :
				if section=="header" and headerlib :library.append(line)
				elif section=="var" and varlib :library.append(line)
				elif section=="gui" and guilib :library.append(line)
				elif section=="function" and funclib :
						library[1][funcnum]=str(library[1][funcnum])+line+"\n"

		# library header
		if section=="header" :
			fields=["author",["all"],"name","comments","version","date",False]
			for line in library :
				args=line.split("=")
				if args[0]=="libName" : fields[0]=str(args[1])
				elif args[0]=="libAuthor" : fields[1]=str(args[1])
				elif args[0]=="libCat" :
					fields[2]=args[1].split(",")
					#print fields[2]
					for id,thiscat in enumerate(fields[2]):
						#append lib to cat:
						new=True
						newlibfield=[fields[0],[D(True),111,"","","toggle"],[D(0.5),111,"","","slider",0,1]]
						for id2,cat in enumerate(CATS) :
							# this category already exist
							if cat[0]==thiscat :
								for c in CATS[id2][2] :
									# this lib already exist in this category
									if c[0]==fields[0] :
										new=False
										break
								# add current library in the existing category
								if new :CATS[id2][2].append(newlibfield)
								new=False
						# create this category & its filters + current library
						if new :
							if len(CATS)==1:but=True
							else:but=False
							CATS.append([ thiscat,
							[[D(True),111,"","","toggle"],
							[D(but),1000+len(CATS),thiscat,"","toggle"],
							[D(1.0),111,"use","","slider",0.0,1.0],
							[D(10),111,"minfloor","","slider",1,1000],
							[D(20),111,"maxfloor","","slider",1,1000],
							[D(10),111,"minsize","","slider",1,1000],
							[D(20),111,"maxsize","","slider",1,1000]
							],
							[newlibfield] ])
							#[[D(True),111,"","","toggle"],[D(but),1000,thiscat,"","toggle"]]
				elif args[0]=="libComment" :fields[3]=args[1].split(";")
				elif args[0]=="libVersion" : fields[4]=str(args[1])
				elif args[0]=="libDate" : fields[5]=str(args[1])
				elif args[0]=="libGui" : fields[6]=findType(args[1])
			#if thisLib<>"none" :
			#print fields
			BLIB[thisLib][0]=fields
			#print BLIB
		# library variables
		elif section=="var" :
			BLIB[thisLib].append([])
			for line in library :
				args=line.split(";")
				varlib=args[0].split("=")
				varlibname=varlib[0]			# variable library name
				var=findType(varlib[1])
				args[0]=D(var)					# 0 default value
				args[1]=int(args[1])			# 1 event
												# 2 button name
												# 3 tip
												# 4 button type
												# 5... arguments
				if args[4]=="slider" :
					args[5]=findType(args[5])
					args[6]=findType(args[6])
				args.append(varlibname)
				BLIB[thisLib][1].append(args)
				#print "ici %s"%(BLIB[thisLib][1])
		# library gui
		elif section=="gui" :
			tab=0
			BLIB[thisLib].append([["common",[]]])
			for line in library :
				if line[0:4]=="<tab" :
					tabname=line[5:line.index(">")]
					tab +=1
					BLIB[thisLib][2].append([tabname,[]])
				else :BLIB[thisLib][2][tab][1].append(line)
			tab=len(BLIB[thisLib][2])
			if tab>0 :
				# this lib asked for tabs
				for i,t in enumerate(BLIB[thisLib][2]):
					BLIB[thisLib][2][i][0]=[D(False),1100+i,t[0],"","toggle"]
			#empty sections cosmetic
			for i,t in enumerate(BLIB[thisLib][2]):
				if len(BLIB[thisLib][2][i][1])<>0 :
					BLIB[thisLib][2][i][0][0].val=True
					break
		#library function
		elif section=="function" :
			id=-1
			for i,lib in enumerate(library[0]) :
				if lib=="generic" :id=i
				elif lib==cat : id=i
			if id==-1 : 
				dprint("'%s' category has no corresponding function in %s"%(cat, subPath.split(sys.sep)[-1]))
				dprint("switch to first function available")
				bell()
				id=0
			createfunc="def %s(listBuilding,pos,bsz,ffloor,floor,lod,around):"%(funcexecname)+"\n"
			library[1][id]=createfunc+"\tMYSELF='"+funcexecname+"'\n"+library[1][id]
			exec library[1][id] in globals()
				
		
	#  include preset
	elif type=="set":
		global MGRM, MLIB, MPRM, MGMM, MPMM, TRPP
		global mapMatList, imageUVlist
		# read .set
		common=True
		library=[[],[]]
		i=-1
		for line in file.readlines():
			line=chomp(line)
			if len(line)>0:
				if line[0]=="<" :
					libname=line[1:line.index(">")]
					common=False
					i +=1
					library[0].append(libname)
					library[1].append([])
				elif common : exec(line) in globals()
				else : library[1][i].append(line)
		file.close()
		#imageUVrefresh()
		#materialRefresh("maps_")
		
		# map preset parameters to internal variables
		guiValues = open(path+"/bin/"+"presetIN.py", "r")
		oldSet=False
		missings=[]
		for line in guiValues.readlines():
			
			try : exec(chomp(line)) in globals()
			except :
				missings.append(line)
				oldSet=True 
			TRPP=[D(False),no]
		if oldSet :
			dprint("some missing values in this set were replaced by default ones :",level=5)
			bell()
			for m in missings :
				print m
		dprint("load set '%s'.",(name))
		guiValues.close()
		
		# menu ground map - list of images in UV editor for street map
		idmap=menuIndex("imageUVlist",groundUvMapName)
		if idmap==-1:
			dprint("'"+groundUvMapName+"' image not found in UV editor. switch to "+imageUVlist[0]+".")
			bell()
			idmap=0
		MGRM=[D(idmap),no,menuList("imageUVlist")]
		
		idmap=menuIndex("imageUVlist",buildingUvMapName)
		if idmap==-1:
			dprint("'"+buildingUvMapName+"' image not found in UV editor. switch to "+imageUVlist[0]+".")
			bell()
			idmap=0
		MPRM=[D(idmap),19,menuList("imageUVlist")]


		idmat=menuIndex("mapMatList",groundMatMapName)
		if idmat==-1:
			dprint("'"+groundMatMapName+"' ground material not found. switch to grd_default")
			bell()
			idmat=menuIndex("mapMatList","grd_default")
		MGMM=[D(idmat),no,menuList("mapMatList")]	
		idmat=menuIndex("mapMatList",buildingMatMapName)
		if idmat==-1:
			dprint("'"+buildingMatMapName+"' building material not found. switch to pop_default")
			bell()
			idmat=menuIndex("mapMatList","pop_default")
		MPMM=[D(idmat),no,menuList("mapMatList")]

		# building lib from set
		try :
			test=buildingLibCategory
			blc=True
		except : blc=False
		if blc :
			# init : disable all categories and libraries use 
			for i,c in enumerate(CATS) :
				c[1][0][0].val=False
				for l in c[-1] : l[1][0].val=False
			# map .set values to internal vars
			for c in buildingLibCategory :
				for i,c2 in enumerate(CATS) :
					if c2[0]==c[0] :
						CATS[i][1][0][0].val=True # this cat is on
						# filter config
						if c[0]<>"default" :
							CATS[i][1][2][0].val=c[1]	# chances to use
							CATS[i][1][3][0].val=c[2]	# min height
							CATS[i][1][4][0].val=c[3]	# max height
							CATS[i][1][5][0].val=c[4]	# min size
							CATS[i][1][6][0].val=c[5]	# max size
						# associated lib. to this cat.
						for libname,var in c[-1] :
							for j,l in enumerate(CATS[i][-1]) :
								#print libname,l[0]
								if libname==l[0] :
									#print CATS[i][-1][j]
									CATS[i][-1][j][1][0].val=True	# this lib is on
									CATS[i][-1][j][2][0].val=var	# chances to use
						# libraries arguments
							#if len(library)> 0 :
							try : 
								k=library[0].index(libname)
								mapvar=True
							except : mapvar=False
							if mapvar :
								for k2,l2 in enumerate(BLIB) :
									if libname==l2[0][0] : break
								for args in library[1][k] :
									args=args.split("=")
									varname=args[0]
									val=findType(args[1])
									for k3,l2 in enumerate(BLIB[k2][1]) :
										if varname==l2[-1] :
											BLIB[k2][1][k3][0].val=val
											#print varname,BLIB[k2][1][k3]
											break

		idlib=menuIndex("libs","simple cube")#buildingLibraryName)
		if idlib==-1:
			dprint("'"+buildingLibraryName+"' library not found in libraries directory. switch to 'simple cube'.")
			bell()
			idlib=menuIndex("libs","simple cube")
		MLIB=[D(idlib),20,menuList("libs")]
		
	elif type=="lang":
		global TIPS
		for line in file.readlines():
			line=chomp(line)
			if len(line)>0 and line[0]<>"#":
				var=line.split("=")
				if var[0][-3:]=="_tt" and TIPS[0].val==False :
					exec(var[0]+"=''") in globals()
				else : exec(line) in globals()
		file.close()
	# include generic
	else :
		dprint("generic line by line exec for %s",(file),level=4)
		for line in file.readlines():
			exec(chomp(line)) in globals()
		file.close()	
	dprint("done.",level=4)
# linux/windows/mac eol trick + seems to solve some unicode issue with fr (or dutch I guess..) 
def chomp(line):
  if line.endswith('\r\n'):return line[:-2]
  elif line.endswith('\r') or line.endswith('\n'):return line[:-1]
  else:return line
# config & default  libraries
def config(todo="init") :
	global libs, presets, locales, path, LANG, MSET, MLIB, MGRM, MPRM, MGMM, MPMM, TLIM, TLIMB
	global mapMatList, imageUVlist, none_text,libPath,BLIB,CATS, MGSC, MBSC
	dprint("refresh/init config : %s",todo,4)
	# currently used lang and lib. id will perhaps change of id because of an update, so we need the string
	if todo<>"init" :
		lang=locales[LANG[0].val]
		set=presets[MSET[0].val]
		lib=libs[MLIB[0].val]
		groundUvMapName=imageUVlist[MGRM[0].val]
		buildingUvMapName=imageUVlist[MPRM[0].val]
		groundMatMapName=mapMatList[MGMM[0].val]
		buildingMatMapName=mapMatList[MPMM[0].val]
		if TLIM[0].val==none_text:TLIM[0].val=""
		if TLIMB[0].val==none_text:TLIMB[0].val=""
		groundSourceType=MGSC[0].val
		buildingSourceType=MBSC[0].val
	# parse cityConfig.cfg (MUST be in the same dir. than city.py)
	if todo=="init" :
		global TDBB,SDGL,TDGO,TIPS,BEEP
		dprint("load configuration...")
	 	include("cityConfig","cfg")
		lang=myLanguage
		set=myPreset
		TDBB=[D(debugButtons),no,"","","toggle"] #39
		TDGO=[D(debugToFile),no,"","","toggle"]
		SDGL=[D(debugCore),no,"","","toggle"]
		TIPS=[D(tips),46,"","","toggle"]
		BEEP=[D(beep),no,"","","toggle"]
	# library refresh
	if todo=="init" or todo=="lib" or todo=="cat" :
		dprint("read libraries...")
		libs=dirContent(path+"/libraries","dir")
		try:del libs[libs.index("common")]
		except ValueError: pass
		llibs=[]
		if todo=="init"or todo=="cat" :
			BLIB=[]
			CATS=[[ "default",[[D(True),111,"","","toggle"],[D(False),1000,"default","","toggle"]],	[] ]]
		# display library information and configuration
		if todo=="lib": 
			idlib=menuIndex("libs",lib) #a library has perhaps been added/deleted meanwhile, so id/name match has perhaps changed
			if idlib==-1:
				dprint("'"+lib+"' library not found in libraries directory.")
				bell()
				idlib=0 #menuIndex("libs","simple cube")
			MLIB=[D(idlib),20,menuList("libs")]
			libReset()
			libPath="libraries"+sys.sep+libs[MLIB[0].val]+sys.sep
			for l in BLIB :llibs.append(l[0][0])
				
		if libs<>llibs :
			# create/update infos about installed libs
			for l in libs :
				if (l in llibs)==False :
					BLIB.append([[l]])
					libPath="libraries"+sys.sep+l+sys.sep
					include("building","lib",libPath,"header")
					# lib has a gui
					if BLIB[-1][0][6]==True : 
						include("building","lib",libPath,"var")
						include("building","lib",libPath,"gui")
			# delete infos about a just deleted (or name edited) lib
			if len(llibs) > len(libs) :
				for i,l in enumerate(llibs) :
					if (l in libs)==False :
						del BLIB[i]
						for j,c in enumerate(CATS):
							for k,lc in enumerate(c[2]) :
								if lc[0]==l :
									del CATS[j][2][k]
	# language
	if todo=="init" or todo=="lang" :
		dprint("read languages...")
		localeFiles=dirContent(path+"/locale",".lang")
		locales=[]
		for s in localeFiles :locales.append(s[0:-5])
		# try to find the language cityconfig asked for, else uk.
		idlang=menuIndex("locales",lang)
		if idlang==-1 :
			dprint("'"+lang+"' language not found in locale directory. switch to english.")
			bell()
			idlang=menuIndex("locales","uk")
		include(locales[idlang],"lang")
		LANG=[D(idlang),46,menuList("locales")]
		if todo=="lang" :
			MBSC=[D(buildingSourceType),no,buildingHeightsSource_list]	#  menu  building  source
			MGSC=[D(groundSourceType),no,groundSource_list]				# menu  ground source

	# image in uv editor list (ground map)
	if todo=="init" or todo=="map" :
		dprint("read uv editor images...")
		imageUVrefresh()
		"""
		idmap=menuIndex("imageUVlist",groundUvMapName)
		if idmap==-1:
			dprint("'"+groundUvMapName+"' image not found in UV editor. switch to "+imageUVlist[0]+".")
			idmap=0
		MGRM=[D(idmap),19,menuList("imageUVlist")]
		idmap=menuIndex("imageUVlist",buildingUvMapName)
		if idmap==-1:
			dprint("'"+buildingUvMapName+"' image not found in UV editor. switch to "+imageUVlist[0]+".")
			idmap=0
		MPRM=[D(idmap),19,menuList("imageUVlist")]
		"""
	# read maps materials
	if todo=="init" or todo=="set" or todo=="matmaps" :
		dprint("read maps materials...")
		#materialRefresh(["map_pop"],["maps_"])
		#materialRefresh(["map_grd"],["maps_"])
		materialRefresh("maps_")
		if todo=="matmaps" :
			idmat=menuIndex("mapMatList",groundMatMapName)
			if idmat==-1:
				dprint("'"+groundMatMapName+"' ground material not found. switch to grd_default")
				bell()
				idmat=menuIndex("mapMatList","grd_default")
			MGMM=[D(idmat),no,menuList("mapMatList")]	
			idmat=menuIndex("mapMatList",buildingMatMapName)
			if idmat==-1:
				dprint("'"+buildingMatMapName+"' building material not found. switch to pop_default")
				bell()
				idmat=menuIndex("mapMatList","pop_default")
			MPMM=[D(idmat),no,menuList("mapMatList")]
		
	# presets
	if todo=="init" or todo=="set" or todo=="refreshset":
		if todo=="init" : dprint("read presets directory...")
		else : dprint("refresh presets directory...",level=4)
		presetFiles=dirContent(path+"/presets",".set")
		presets=[]
		for s in presetFiles :presets.append(s[0:-4])
		# try to find the preset, else default one.
		idset=menuIndex("presets",set)
		if idset==-1 :
			dprint("'"+set+"' set not found in presets directory. switch to default.")
			bell()
			set="default"
			idset=menuIndex("presets",set)
		MSET=[D(idset),30,menuList("presets")]	# buildings LOD library + loader
		if todo<>"refreshset" :
			dprint("read '%s' set..."%(set))
			libReset()
			include(presets[idset],"set") # (re)load it
			libPath="libraries"+sys.sep+libs[MLIB[0].val]+sys.sep
			include("building","lib",libPath,"header")

		
def saveConfig() :
	global path, locales#, LANG
	dprint("save configuration.")
	cityConfig = open(path+"cityConfig.cfg", "w")
	cityConfig.write('myPreset="'+presets[MSET[0].val]+'"\n')
	cityConfig.write('myLanguage="'+locales[LANG[0].val]+'"\n')
	cityConfig.write('debugButtons='+str(bool(TDBB[0].val))+'\n')
	cityConfig.write('debugCore='+str(SDGL[0].val)+'\n')
	cityConfig.write('debugToFile='+str(bool(TDGO[0].val))+'\n')
	cityConfig.write('beep='+str(bool(BEEP[0].val))+'\n')
	cityConfig.write('tips='+str(bool(TIPS[0].val))+'\n')
	cityConfig.close()
	
def crumb(float,ask=False) :
	crumb=float-int(float)
	if ask : return float-int(float)
	else :
		if crumb >0 : return 1
		else : return 0
def nearInt(float) :
	crumb=float-int(float)
	if crumb >= 0.5 : crumb=1
	else : crumb=0
	return int(float)+crumb
	
# print to console if debug
def bell() :
	global beep,debugCore
	if beep and debugCore<=5 : print chr(7)
def dprint(string="",args="",level=5,pxart=False) :
	if pxart :
		string=string.replace('O', chr(178))
		string=string.replace('o', chr(177))
		string=string.replace('.', chr(176))

	global debugCore
	global debugOut
	if level >= debugCore :
		if string=="<END>": string="--------------------------------------------------------"
		if args<>"":exec "print string %(args)"
		else : print string
		
		
# dprint("there's %s buddies, %s and %s",(2,"sam","nicole"),5)
# dprint()
# dprint("simple",level=6)
# dprint("more simple") #level 5 by default
# debug levels (wip-something like) :
# 0 - maximum of stuff
# 2 - "i'm here" from significant subroutines
# 3 - 
# 4 - warnings
# 5 - main information - file system operation, main steps in the build loop.
# 6 - nothing but title.

def menuList(list,nb="",title=""):
	exec ("global "+list)
	exec ("glist="+list)
	n=0
	if title=="" :menulist=""
	else:menulist=title+" %t"
	for item in glist :
		#if len(nb)>0 :
		if nb<>"" and n==nb :return item
		else : menulist +="|%s" %(item) + " %x" + str(n)
		n +=1
	return menulist

def menuIndex(list,string) :
	exec ("global "+list)
	exec ("glist="+list)
	try:id = glist.index(string)
	except ValueError:id = -1
	return id

def imageUVrefresh():
	global imageUVlist
	imageUVlist=[]
	for image in Image.Get():
		if image.getName()<>"Render Result" and image.getName()<>"popMap" and image.getName()<>"groundMap" and image.getName()<>"Viewer Node" :
			try :
				test=image.size[0]
				imageUVlist.append(image.getName())
			except:dprint("problem with imageUV '%s' in imageUVrefresh(). excluded.",(image.getName()),3)
def materialRefresh(filter=""):
	global mapMatList
	mapMatList=[]
	
	for mat in Material.Get() :
		if filter=="" : mapMatList.append(mat.name)
		elif mat.name[0:len(filter)]==filter : 
			mapMatList.append(mat.name[len(filter):])
	#if filters[0]="map_pop" : MatList[0]=mList
	#else : MatList[1]=mList

# needed by the area determination. check for constructible units in the main grid. return new x y and a status
def move(grid,pos,width,axis='x',ok="undefined") :
	#global grid
	try :
		x=int(pos[0])
		y=int(pos[1])
	except :
		x=y=pos
	if x>len(grid)-1 or y>len(grid[0])-1 or x<0 or y<0 : return -1,False
	
	offset=0
	coord=0
	status=""
	if ok=="undefined" :
		if axis=='x' : sign=1
		else : sign=1
		if grid[x][y]<=0 : 
			#print "one"
			ok=True
			#Width=-abs(Width)*sign
		else:
			ok=False
			width=abs(width)*sign
	
	if axis=='x' :
		yDir=0
		if width>0 : xDir=1
		elif width<0 : xDir=-1
		else : xDir=0
	else :
		xDir=0
		if width>0 : yDir=1
		elif width<0 : yDir=-1
		else : yDir=0	
	#if x<8 : print "check from %s %s on %s %s (width : %s)" %(x,y,axis,xDir+yDir,width)
	if xDir<>0 :
		while offset<>width :
			offset +=xDir
			if x+offset<0 or x+offset > len(grid)-1 :
				offset -=xDir
				break
			if ok : 
				if grid[x+offset][y]>=1 :
					status=True #"ok (wall next)"
					offset -=xDir
					break
			elif grid[x+offset][y]<=0 :
					status=True #"ok (area)"
					break
		if status=="" : 
			if ok : status=True #"ok"
			else : 
				status=False #"failed"
				offset=0
		coord=x+offset

	if yDir<>0 :
		while offset<>width  :
			offset +=yDir
			if y+offset<0 or y+offset > len(grid[0])-1 :
				offset -=yDir
				break
			if ok : 
				if grid[x][y+offset]>=1 :
					status=True #"ok (wall next)"
					offset -=yDir
					break
			elif grid[x][y+offset]<=0 :
					status=True #"ok (area)"
					break
		if status=="" : 
			if ok : status=True #"ok"
			else : 
				status=False #"failed"
				offset=0
		coord=y+offset
	return coord,status

	
def text(string,x,y,size="normal"):
	sizefont=["tiny","small","normal","large"]
	width=Window.GetAreaSize()[0]
	height=Window.GetAreaSize()[1]
	sizeratio = 0
	if width<250 or height<260 : sizeratio = -3
	elif width<290 or height<300 : sizeratio = -2
	elif width<340 or height<600 : sizeratio = -1
	s=sizefont.index(size)
	size=sizefont[max(0,s+sizeratio)]
	Blender.BGL.glRasterPos2i(x,y+5)
	gTextTitle = Blender.Draw.Text(string,size)
	return gTextTitle

def makeBox(x ,y ,w ,h ,r ,g ,b ):
	glColor3f(r, g, b)
	glRecti(x, y, w, h)
	glColor3f(0.0, 0.0, 0.0)

def button(BUT,xb,yb,xb2,yb2,text=".",tip="."):
	#print BUT[0].val
	if BUT[4]=="slider":
		if text=="." : text=BUT[2]
		if tip=="." : tip=BUT[3]
		return DS(text,BUT[1],xb,yb,xb2,yb2,BUT[0].val,BUT[5],BUT[6],0,tip)
	elif BUT[4]=="toggle":
		if text=="." : text=BUT[2]
		if tip=="." : tip=BUT[3]
		return DT(text,BUT[1],xb,yb,xb2,yb2,BUT[0].val,tip)
	#else :print "rien"
def gui():
	global path, locales, MLIB, MGRM, MSET, failed, MGSC, MBSC, pastLang, escTrick,refreshConfig, imageUVlist, BLIB
	global libComment,thisLib,buildingSourceType,groundSourceType, sceon,height,width
	# glob groundMapSource
	if refreshConfig[0] :
		config(refreshConfig[1])
		if refreshConfig[1]=="cat" : config("set")
		#if refreshConfig[1]=="init" or refreshConfig[1]=="lang" :
			#MBSC=[D(buildingSourceType),no,buildingHeightsSource_list]	# menu  building source
		if TLIM[0].val=="" : TLIM[0].val=none_text
		if TLIMB[0].val=="" : TLIMB[0].val=none_text
		refreshConfig[0]=False
		if refreshConfig[1]=="init" :
			sceon=True
			height=0
			width=0

	# gui scaling and bling-bling 
	if sceon :
		height +=Window.GetAreaSize()[1]/30
		#width +=Window.GetAreaSize()[0]/15
		pos=Window.GetAreaSize()[1]/2+height/2
		if height>Window.GetAreaSize()[1] :
			sceon=False
			bell()
	if sceon==False :
		height=Window.GetAreaSize()[1]
		pos=height-25
	width=Window.GetAreaSize()[0]-20		
	#needed lines
	lines=34
	sy=height/lines
	# gui display vars :
	BGL.glClearColor(0.7,0.7,0.7,1)
	BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
	px,py=10,10 # position x, position y
	tx=10 # offset
	
	#width=348 #sx*2+tx*3 - sx gives tot size, then tot size gives sx.. :-/

	sy3=sy*3/2
	
	col=width/2+tx/2
	
	col1=px+tx
	col22=px+col
	sx=width/2-tx-tx/2
	
	col23=col1+(width-tx)/3
	col33=col1+((width-tx)/3)*2
	sx3=(width-tx*3)/3

	col24=col1+(width-tx)/4
	col34=col1+((width-tx)/4)*2
	col44=col1+((width-tx)/4)*3
	sx4=(width-tx*4)/4	
	
	if escTrick>0 :	escTrick -=1 # prevent esc quit after an imageselector or a fileselector call
	if MGSC[0].val==4 : 
		try : groundMapSource=Image.Load(path+"tmp/groundMap.png")
		except :
			dprint("groundMap does not exist. create from another source first")
			MGSC[0].val=0
	if MGSC[0].val==0:groundMapSource=Image.Get(imageUVlist[MGRM[0].val])
	elif MGSC[0].val==1 and TLIM[0].val <> none_text :
		groundMapSource=Image.Load(TLIM[0].val)

	if MBSC[0].val==4 :
		try : popMapSource=Image.Load(path+"tmp/popMap.png")
		except :
			dprint("popMap does not exist. create from another source first")
			MBSC[0].val=3
	if MBSC[0].val==0 : popMapSource=Image.Get(imageUVlist[MPRM[0].val])
	elif MBSC[0].val==1 and TLIMB[0].val <> none_text : popMapSource=Image.Load(TLIMB[0].val)

	# MAIN PANEL
	
	makeBox(px, pos+sy+5, px+width, pos-sy*lines, 0.753, 0.753, 0.753)
	makeBox(px, pos+sy+5, px+width, pos-3, 0.647, 0.647, 0.647)
	BGL.glColor3f(7,7,7)
	titleoffs=text("Suicidator City Engine ",col1,pos,"large")
	text(sceVersion+" - 2.48a",col1+titleoffs,pos,"small")
	LANG[0]=DM(LANG[2],LANG[1],col22+sx-40,pos,40,sy,LANG[0].val,language_tt)
	BGL.glColor3f(0,0,0)
	
	
	# TODO : lib or prim error traceback
	if 2==4 : #failed<>"" : primitives or libraries errors in the gui
		pos -=sy*4
		makeBox(col1, pos, px+width-tx, pos-sy*7, 0.812, 0.5, 0.5)
		Blender.BGL.glRasterPos2d(col1*2,pos)
		gTextTitle2 = Blender.Draw.Text(failed)
	else :
		# MAIN information Panel
		pos -=sy
		makeBox(col1-2, pos+5, px+width-tx+4, pos-sy*7, 0.812, 0.812, 0.812)
		pos -=sy
		text(citydim_text,col1*2,pos)
		
		if MGSC[0].val==0 or (MGSC[0].val==1 and TLIM[0].val<>none_text) or MGSC[0].val==4 :
			xCityGround=SGDS[0].val*groundMapSource.size[0]
			yCityGround=SGDS[0].val*groundMapSource.size[1]
		elif MGSC[0].val==3 :
			xCityGround=SGDS[0].val*SGDX[0].val
			yCityGround=SGDS[0].val*SGDY[0].val
		else :
			xCityGround=0
			yCityGround=0

		if MBSC[0].val==0 or (MBSC[0].val==1 and TLIMB[0].val<>none_text) or MBSC[0].val==4 :
			xCityPop=SBD[0].val*popMapSource.size[0]
			yCityPop=SBD[0].val*popMapSource.size[1]
		elif MBSC[0].val==3 :
			xCityPop=SBD[0].val*SPMX[0].val
			yCityPop=SBD[0].val*SPMY[0].val
		else :
			xCityPop=0
			yCityPop=0

			
		xCity=max(xCityGround,xCityPop)
		yCity=max(yCityGround,yCityPop)
		pos -=sy
		text("%.1f x %.1f %s" %(xCity,yCity,meter_text),col1*3,pos)
		pos -=sy
		text("%.1f x %.1f %s" %(xCity*SSCL[0].val,yCity*SSCL[0].val,blenderu_text),col1*3,pos)
		pos -=sy
		text(buildingdim_text,col1*2,pos)
		pos -=sy
		maxsz=(nhdf[0].val-1)*SFHM[0].val+SFFH[0].val*SFHM[0].val
		minsz=(nhf[0].val-1)*SFH[0].val+SFFH[0].val*SFH[0].val
		if THWR[0].val :HWR=SHWR[0].val
		else :HWR=maxsz*2
		text("%s %.1f %s - %s %.1f / %.1f" %(highest_text,maxsz,meter_text,width_text,max(maxsz/HWR,SBSMi[0].val),max(maxsz/HWR,SBSMx[0].val)),col1*3,pos)
		pos -=sy
		text("%s %.1f %s - %s %.1f / %.1f" %(lowest_text,minsz,meter_text,width_text,max(minsz/HWR,SBSMi[0].val),max(minsz/HWR,SBSMx[0].val)),col1*3,pos)
		pos -=sy*3
		
		# TABS :
		
		TMn[0]=DT(mainTab_text,TMn[1],col1,pos,(width-tx*4)/4,sy*3/2,TMn[0].val,mainTab_tt)
		TB[0]=DT(buildings_text,TB[1],col1+(width-tx)/4,pos,(width-tx*4)/4,sy*3/2,TB[0].val,buildingTab_tt)
		TG[0]=DT(ground_text,TG[1],col1+2*((width-tx)/4),pos,(width-tx*4)/4,sy*3/2,TG[0].val,groundTab_tt)
		TMp[0]=DT(inputTab_text,TMp[1],col1+3*((width-tx)/4),pos,(width-tx*4)/4,sy*3/2,TMp[0].val,inputTab_tt)
		
		pos -=sy*2

		# 'MAIN' section
		if TMn[0].val :
			pos -=sy
			ppos=pos
			text(presets_text,col1,pos)
			pos -=sy
			MSET[0]=DM(MSET[2],MSET[1],col1,pos,sx-16,sy,MSET[0].val,presetsLoad_tt)
			BPSR[0]=DPB(reloadSmall_text,BPSR[1],col1+sx-16,pos,16,sy,presetRefresh_tt)
			pos -=sy
			BPXX[0]=DPB(del_text,BPXX[1],col1,pos,sx/3,sy,deletePreset_tt)
			BPSV[0]=DPB(save_text,BPSV[1],col1+sx/3,pos,sx/3,sy,savePreset_tt)
			BPSA[0]=DPB(saveAs_text,BPSA[1],col1+2*(sx/3),pos,sx/3,sy,saveAsPreset_tt)
			pos=ppos
			text(random_text,col22,pos)
			pos -=sy	
			TUSD[0]=DT(replay_text,TUSD[1],col22,pos,sx,sy,TUSD[0].val,replay_tt)
			if TUSD[0].val :
				ppos=pos
				pos -=sy
				SSED[0]=DS(seed_text,SSED[1],col22,pos,sx,sy,SSED[0].val,1,500,0,seedSlider_tt)
				pos=ppos
			pos -=sy*4
			ppos=pos
			text(defaultLangAndCity_text,col1,pos)
			pos -=sy
			BEEP[0]=button(BEEP,col1,pos,sx/3,sy,beep_text,beep_tt)
			TIPS[0]=button(TIPS,col1+sx/3,pos,sx/3,sy,tips_text,tips_tt)
			TDBB[0]=button(TDBB,col1+(sx/3)*2,pos,sx/3,sy,debugButtons_text,debugButtons_tt)
			pos -=sy
			BCSV[0]=DPB(save_text,BCSV[1],col1,pos,sx/2,sy,saveConfig_tt)
			BCLD[0]=DPB(load_text,BCLD[1],col1+sx/2,pos,sx/2,sy,loadConfig_tt)
			#TDBB[0]=DT("D",TDBB[1],col1+sx-16,pos,16,sy,TDBB[0].val,debugButtons_tt)
			if TDBB[0].val :
				#pos=ppos
				pos -=sy*3/2
				text(debug_text,col1,pos)
				pos -=sy
				SDGL[0]=DS("",SDGL[1],col1,pos,sx/2,sy,SDGL[0].val,1,6,0,debug_tt)
				TDGO[0]=DT(debugToFile_text,TDGO[1],col1+sx/2,pos,sx/2,sy,TDGO[0].val,debugToFile_tt)
		# 'BUILDING' section
		if TB[0].val :
			TBDM[0]=DT(properties_text,TBDM[1],col1,pos,(width-tx*3)/3,sy*3/2,TBDM[0].val,properties_tt)
			TBLD[0]=DT(libraries_text,TBLD[1],col23,pos,sx3,sy3,TBLD[0].val,libraries_tt)			
			TBLC[0]=DT(libraryConfig_text,TBLC[1],col33,pos,sx3,sy3,TBLC[0].val,libraryConfig_tt)			
			pos -=sy*2
			# Properties
			if TBDM[0].val :
				makeBox(col1-2, pos+sy*3/2, px+width-tx+4, pos-sy*12, 0.812, 0.812, 0.812)
				#if MGSC[0].val<>2 and TMGD[0].val==False :
				#	TRGP[0]=DT(groundInfluence_text,TRGP[1],col1,pos,sx,sy,TRGP[0].val,groundInfluence_tt)
				pos -=sy*3/2
				text(floors_text,col1,pos)
				text(floor_text,col22,pos)
				pos -=sy
				nhf[0]=button(nhf,col1,pos,sx,sy,minimum_text,floorMinNumber_tt)

				SFFH[0]=button(SFFH,col22,pos,sx,sy,firstFloor_text,firstFloor_tt)
				pos -=sy
				nhdf[0]=DS(maximum_text,nhdf[1],col1,pos,sx,sy,nhdf[0].val,1,1000,0,floorMaxNumber_tt)
				SFH[0]=DS(minimum_text,SFH[1],col22,pos,sx,sy,SFH[0].val,2,5,0,floorMinHeight_tt)
				pos -=sy
				SBHV[0]=DS("Var  ",SBHV[1],col1,pos,sx,sy,SBHV[0].val,0,1,0,floorNumberVariation_tt)
				SFHM[0]=DS(maximum_text,SFHM[1],col22,pos,sx,sy,SFHM[0].val,2,5,0,floorMaxHeight_tt)
				pos -=sy*2
				text(width_text+" :",col1,pos)
				text(interval_text,col22,pos)
				pos -=sy
				THWR[0]=button(THWR,col1,pos,25,sy,on_text,HWratioButton_tt)
				SHWR[0]=DS(HWratio_text,SHWR[1],col1+25,pos,sx-25,sy,SHWR[0].val,1,10,0,HWratio_tt)
				#SISMi[0]=DS(minimum_text,SISMi[1],col22,pos,sx,sy,SISMi[0].val,0.0,50.0,0,SISMi_tt)
				SISMi[0]=button(SISMi,col22,pos,sx,sy,minimum_text,buildingIntMinSpace_tt)
				pos -=sy
				SBSMi[0]=DS(minimum_text,SBSMi[1],col1,pos,sx,sy,SBSMi[0].val,3.0,100.0,0,buildingMinSize_tt)
				SISMx[0]=button(SISMx,col22,pos,sx,sy,maximum_text,buildingIntMaxSpace_tt)
				pos -=sy
				SBSMx[0]=button(SBSMx,col1,pos,sx,sy,maximum_text,buildingMaxSize_tt)
				SCAT[0]=DS(scattering_text,SCAT[1],col22,pos,sx,sy,SCAT[0].val,0.0,1.0,0,scattering_tt)
				
				pos -=sy*2
				TBGS[0]=DT(alignBuildingToRoad_text,TBGS[1],col1,pos,sx,sy,TBGS[0].val,alignBuildingToRoad_tt)
				TBGG[0]=DT(alignVariations_text,TBGG[1],col22,pos,sx,sy,TBGG[0].val,alignVariations_tt)
				#SBBV[0]=DS("Blend  ",SBBV[1],col22,pos,sx,sy,SBBV[0].val,0.0,1.0,0,SBBV_tt)
				#pos -=sy
				
				#SRBL_tt="Low = buildings will be grid-aligned. High = buildings locations are randomized."
				#SRBL[0]=DS("Var  ",SRBL[1],col22,pos,sx,sy,SRBL[0].val,0.0,1.0,0,SRBL_tt)
				#SNBP[0]=DS("B/px   ",SNBP[1],col1,pos,sx,sy,SNBP[0].val,1,10,0,SNBP_tt)
				#SNBP_tt="The higher, the more buildings MAY be created per pixel of the population map = building "
				#SNBP_tt+="Subdivision. Shorter buildings are subdivided more times."
			
			# Dispatcher - Libraries blender
			elif TBLD[0].val :
				col=col1
				i=len(CATS)/2
				if i-int(i)>0 : i=int(i)+1
				makeBox(col1-2, pos+sy*3/2, px+width-tx+4, pos-sy*i-sy/2, 0.812, 0.812, 0.812)
				text(availableCats_text,col1,pos)
				BCTR[0]=DPB(reloadSmall_text,BCTR[1],width-16,pos,16,sy,reloadCategories_tt)
				pos -=sy
				for id,thiscat in enumerate(CATS) :
					if id>0 :
						CATS[id][1][1][0]=button(CATS[id][1][1],col,pos,sx-25,sy) # activate cat config and details
						CATS[id][1][0][0]=button(CATS[id][1][0],col+sx-25,pos,25,sy,on_text,activateCat_tt) # activate cat
						if col==col1 : col=col22
						else :
							col=col1
							pos -=sy
					if CATS[id][1][1][0].val==True : currentCat=id
				CATS[0][1][1][0]=button(CATS[0][1][1],col,pos,sx,sy) # activate cat config and details
				#CATS[0][1][0][0]=button(CATS[0][1][0],col+sx-25,pos,25,sy,on_text,activateCat_tt) # activate cat				
				pos -=sy*2
				if CATS[currentCat][1][0][0].val :
					# define this category
					#text(defineCat_text+" "+CATS[currentCat][0]+" :",col1,pos)
					if currentCat==0 :
						makeBox(col1-2, pos+sy, px+width-tx+4, pos-sy*1-sy/2, 0.812, 0.812, 0.812)
						text(defaultCatwarn1_text,col1,pos)
						pos -=sy
						text(defaultCatwarn2_text,col1,pos)
					else :
						makeBox(col1-2, pos+sy, px+width-tx+4, pos-sy*3-sy/2, 0.812, 0.812, 0.812)
						text(defineCat_text+" "+CATS[currentCat][0]+" :",col1,pos)
						CATS[currentCat][1][2][0]=button(CATS[currentCat][1][2],col33,pos+2,sx3,sy-5,"",categoryUseProba_tt) # proba. to use
						pos -=sy
						text(floorFilter_text,col1,pos)
						pos -=sy
						CATS[currentCat][1][3][0]=button(CATS[currentCat][1][3],col1,pos,sx,sy,"min ",floorFilter_tt) #  floor filter
						pos -=sy
						CATS[currentCat][1][4][0]=button(CATS[currentCat][1][4],col1,pos,sx,sy,"max ",floorFilter_tt) # 
						pos +=sy*2
						text(sizeFilter_text,col22,pos)
						pos -=sy
						CATS[currentCat][1][5][0]=button(CATS[currentCat][1][5],col22,pos,sx,sy,"min ",sizeFilter_tt) # size filter
						pos -=sy
						CATS[currentCat][1][6][0]=button(CATS[currentCat][1][6],col22,pos,sx,sy,"max ",sizeFilter_tt) # 					
					pos -=sy*2
					# available libs for this category
					i=len(CATS[currentCat][2])/2
					if i-int(i)>0 : i=int(i)+1
					makeBox(col1-2, pos+sy, px+width-tx+4, pos-sy*+i-sy/2, 0.812, 0.812, 0.812)
					text(availableLibs_text,col1,pos)
					pos -=sy
					i=0
					col=col1
					tcol=col24
					for id,libsincat in enumerate(CATS[currentCat][2]) :
						text(libsincat[0],tcol,pos)	# lib name
						CATS[currentCat][2][id][1][0]=button(CATS[currentCat][2][id][1],col,pos+2,10,sy-5,on_text,activateLib_tt) # activate lib for this cat
						CATS[currentCat][2][id][2][0]=button(CATS[currentCat][2][id][2],col+10,pos+2,sx4-10,sy-5,"",chanceLib_tt) # probability to use
						if col==col1 :
							col=col34
							tcol=col44
						else :
							col=col1
							tcol=col24
							pos -=sy
						i +=CATS[currentCat][2][id][1][0].val
					if i==0:
						CATS[currentCat][2][0][1][0].val=True
						Draw.Redraw()
				else :
					text(activateCatwarn_text,col1,pos)

			# Library config
			elif TBLC[0].val :
				BLBR[0]=DPB(reloadSmall_text,BLBR[1],col1,pos,16,sy,refreshLibraries)
				MLIB[0]=DM(MLIB[2],MLIB[1],col1+16,pos,sx-16,sy,MLIB[0].val,librarySelect_tt)
				thisLib=fromLib(libs[MLIB[0].val])
				header=BLIB[thisLib][0]
				# header : 0 libname, 1 author, 2 categories, 3 comments, 4 version, 5 date, 6 has a gui (bool)
				if type(header[3])==str :
					libComment=[header[3]]
				makeBox(col1-2, pos+sy*3/2, px+width-tx+4, pos-sy*(len(header[3])+2), 0.812, 0.812, 0.812)
				pos -=sy
				cat=""
				if len(header[2])>1 : c_text=categories_text
				else :  c_text=category_text
				for i in range(len(header[2])) : 
					cat +=header[2][i]
					if i<len(header[2])-1 : cat +=', '
					else : cat +='.'
				text(c_text+" : %s"%(cat),col1,pos)
				for comment in header[3] :
					pos -=sy
					text(comment,col1,pos,"small")
				pos -=sy
				text(by_text+" "+header[1]+" "+header[4]+" "+header[5],col1,pos,"small")
				
				# this lib has its own gui
				if header[6] :
					pos -=sy*3/2
					tab=len(BLIB[thisLib][2])
					if tab>1 :
						pos -=sy/2
						# this lib asked for tabs
						if tab<3:tab=3
						tabsx=(width-tx*(tab+2))/(tab+1)
						offset=0
						for i,t in enumerate(BLIB[thisLib][2]):
							if len(BLIB[thisLib][2][i][1])>0 :
								tabcol=col1+((width-tx)/(tab+1))*offset
								BLIB[thisLib][2][i][0][0]=button(BLIB[thisLib][2][i][0],tabcol,pos,tabsx,sy3)
								if BLIB[thisLib][2][i][0][0].val : thisTab=i
								offset +=1
						pos -=sy*3/2
					else : thisTab=0
					var=-1
					for line in BLIB[thisLib][2][thisTab][1] :
						# display current tab
						if line[0]=="*" :
							loc = line.split(",")
							varname=findType(loc[0][1:])
							for var in range(len(BLIB[thisLib][1])) :
								if varname==BLIB[thisLib][1][var][-1] :
									BLIB[thisLib][1][var][0]=button(BLIB[thisLib][1][var],eval(loc[1]),eval(loc[2]),eval(loc[3]),eval(loc[4]))
						else : exec line
					#print BLIB[thisLib]
			#print retrievefromgui("windowCatNb"),type(retrievefromgui("windowCatNb"))
			
		# 'GROUND' section
		if TG[0].val :		
			pos -=sy*2
			# if properties..
			#if MGSC[0].val==1 : TRPP[0].val=True
			#TRPP[0]=DT(buildingInfluence_text,TRPP[1],col1,pos,sx,sy,TRPP[0].val,buildingInfluence_tt)
			pos -=sy*3/2
			ppos=pos
			text(sidewalks_text+" :",col1+35,pos)
			# sidewalks button ON
			if TRPP[0].val==False and MGSC[0].val==2 : TMSW[0].val=False
			TMSW[0]=DT(on_text,TMSW[1],col1,pos,25,sy,TMSW[0].val,createSidewalks_tt) 
			pos -=sy

			if TMSW[0].val :
				SWH[0]=DS(sidewalkHeight_text,SWH[1],col1,pos,sx,sy,SWH[0].val,0.05,0.5,0,sidewalkHeight_tt)
				pos -=sy
				SWMi[0]=DS(sidewalkMinWidth_text,SWMi[1],col1,pos,sx,sy,SWMi[0].val,0.5,10,0,sidewalkMinWidth_tt)
				pos -=sy
				SWMx[0]=DS(sidewalkMaxWidth_text,SWMx[1],col1,pos,sx,sy,SWMx[0].val,0.5,10,0,sidewalkMaxWidth_tt)				
				pos -=sy
				TMTS[0]=DT(tiled_text,TMTS[1],col1,pos,sx,sy,TMTS[0].val,tiled_tt)
				pos -=sy
				TSWS[0]=DT(alignToRoad_text,TSWS[1],col1,pos,sx,sy,TSWS[0].val,alignToRoad_tt)
				if TRPP[0].val==True :
					pos=ppos
					text(method_text,col22,pos)
					pos -=sy
					text("around",col22+tx,pos)
					SWSR[0]=DN("",SWSR[1],col22+70,pos,40,sy,SWSR[0].val,0,100,SWSR_tt)
					text("%",col22+sx-40,pos)
					pos -=sy
					text("align",col22+tx,pos)
					text("%s" %(100-SWSR[0].val),col22+80,pos)
					text("%",col22+sx-40,pos)
					pos -=sy+sy/2
					text("Build around :",col22,pos)
					pos -=sy
					text(buildings_text,col22+tx,pos)
					SWMM[0]=DN("",SWMM[1],col22+70,pos,40,sy,SWMM[0].val,0,100,SWMM_tt)
					text("%",col22+sx-40,pos)
					pos -=sy
					text("Blocks",col22+tx,pos)
					text("%s" %(100-SWMM[0].val),col22+80,pos)
					text("%",col22+sx-40,pos)
					#SWMM_tt=""
					#SWMM[0]=DM(SWMM[2],SWMM[1],px+sx,py+5*sy,sx,sy,SWMM[0].val,SWMM_tt)
					#THWR[0]=button(THWR,col1,pos,25,sy,on_text,THWR_tt)
				if TMTS[0].val==False :
					pos -=sy*3/2
					text(bevelQuality_text,col1,pos)
					pos -=sy
					text(corners_text,col1,pos)
					SWSH[0]=button(SWSH,col24,pos+2,sx4,sy-5,qualitySmall_text,cornersQuality_tt)		# sidewalks Smooth Corners
					pos -=sy
					text(cornersLenght_text,col1,pos)
					SWB[0]=button(SWB,col24,pos+2,sx4,sy-5,"",cornersLenght_tt)			# sidewalks Corners Width
					pos -=sy
					text(borders_text,col1,pos)
					SWSV[0]=button(SWSV,col24,pos+2,sx4,sy-5,qualitySmall_text,bordersQuality_tt)		# sidewalks Smooth Borders

		# 'MAP' section
		if TMp[0].val :
			text(scaleMain_text,col1,pos)
			pos -=sy
			SSCL[0]=DS(scale_text+" ",SSCL[1],col1,pos,sx,sy,SSCL[0].val,0.1,5,0,scaleSlider_tt)
			pos -=sy*3/2
			nextcol=pos
		# buildings
			text(buildings_text+" :",col1,pos)
			pos -=sy
			if MBSC[0].val==0 or ( MBSC[0].val==1 and TLIMB[0].val<>none_text) or MBSC[0].val==4 :
				text("%s x %s px / %s x %s m" %(popMapSource.size[0],popMapSource.size[1],popMapSource.size[0]*SBD[0].val,popMapSource.size[1]*SBD[0].val),col1,pos,"small")
			elif MBSC[0].val==3  :
				text("%s x %s px / %s x %s m" %(SPMX[0].val,SPMY[0].val,SPMX[0].val*SBD[0].val,SPMY[0].val*SBD[0].val),col1,pos,"small")
			elif MBSC[0].val==1 :
				text(noImage_text,col1,pos,"small")
			pos -=sy
			MBSC[0]=DM(MBSC[2],MBSC[1],col1,pos,sx,sy,MBSC[0].val,sourceType_tt) # building source type
			pos -=sy
			if MBSC[0].val==4 :ppos=pos
			else:ppos=pos-sy
			SBD[0]=DS(meterPerPixel,SBD[1],col1,ppos,sx,sy,SBD[0].val,1,100,0,mapScale_tt)
			ppos=pos
			if MBSC[0].val==0 : # image uv
				MPRM[0]=DM(MPRM[2],MPRM[1],col1,pos,sx-16,sy,MPRM[0].val,chooseImage_tt)
				BRFS[0]=DPB(reloadSmall_text,BRFS[1],col1+sx-16,pos,16,sy,refreshList_tt)
				pos -=sy
			elif MBSC[0].val==1 : # image
				TLIMB[0]=DST("", TLIMB[1],col1,pos,sx-16,sy, TLIMB[0].val, 399, pathToFile_tt)
				BLIMB[0]=DPB("L",BLIMB[1],col1+sx-16,pos,16,sy,openImage_tt)
				pos -=sy
			elif MBSC[0].val==3 : # material
				MPMM[0]=DM(MPMM[2],MPMM[1],col1,pos,sx-16,sy,MPMM[0].val,choosemat_tt)
				BMTR[0]=DPB(reloadSmall_text,BMTR[1],col1+sx-16,pos,16,sy,refreshList_tt)
				pos -=sy*2
				SPMX[0]=DS(sizeX_text,SPMX[1],col1,pos,sx,sy,SPMX[0].val,10,1024,0,sizePixelX_tt)
				pos -=sy
				SPMY[0]=DS(sizeY_text,SPMY[1],col1,pos,sx,sy,SPMY[0].val,10,1024,0,sizePixelY_tt)
				pos -=sy
				if TRPM[0].val and TUSD[0].val :
					xbut=sx/4
					TRPS[0]=DT(seed_text,TRPS[1],col1+sx-xbut,pos,xbut,sy,TRPS[0].val,useSeed_tt)
				else:xbut=0
				TRPM[0]=DT(randomOffset_text,TRPM[1],col1,pos,sx-xbut,sy,TRPM[0].val,randomOffset_tt)
			pos -=sy
			STRT[0]=DS(minColor_text,STRT[1],col1,pos,sx,sy,STRT[0].val,0.00,0.8,0,minColor_tt)
			
		#level of detail :
			pos=ppos -6*sy
			text(levelOfDetails_text,col1,pos)
			pos -=sy
			SMDB[0]=DS("",SMDB[1],col1,pos,sx,sy,SMDB[0].val,0,10,0,levelOfDetails_tt)
			
		# ground :
			pos=nextcol
			text(ground_text+ ":",col22,pos)
			pos -=sy
			if MGSC[0].val==0 or ( MGSC[0].val==1 and TLIM[0].val<>none_text) or MGSC[0].val==4 :
				text("%s x %s px / %s x %s m" %(groundMapSource.size[0],groundMapSource.size[1],groundMapSource.size[0]*SGDS[0].val,groundMapSource.size[1]*SGDS[0].val),col22,pos,"small")
			elif MGSC[0].val==3  :
				text("%s x %s px / %s x %s m" %(SGDX[0].val,SGDY[0].val,SGDX[0].val*SGDS[0].val,SGDY[0].val*SGDS[0].val),col22,pos,"small")
			elif MGSC[0].val==1 :
				text(noImage_text,col22,pos,"small")
			pos -=sy
			MGSC[0]=DM(MGSC[2],MGSC[1],col22,pos,sx,sy,MGSC[0].val,sourceType_tt)
			pos -=sy
			if MGSC[0].val==0 : # image uv
				MGRM[0]=DM(MGRM[2],MGRM[1],col22,pos,sx-16,sy,MGRM[0].val,chooseImage_tt)
				BRFS[0]=DPB(reloadSmall_text,BRFS[1],col22+sx-16,pos,16,sy,refreshList_tt)
				pos -=sy
			elif MGSC[0].val==1 : # image
				TLIM[0]=DST("", TLIM[1],col22,pos,sx-16,sy, TLIM[0].val, 399, pathToFile_tt)
				BLIM[0]=DPB("L",BLIM[1],col22+sx-16,pos,16,sy,openImage_tt)
				pos -=sy
			elif MGSC[0].val==3 : # material
				MGMM[0]=DM(MGMM[2],MGMM[1],col22,pos,sx-16,sy,MGMM[0].val,choosemat_tt)
				BMTR[0]=DPB(reloadSmall_text,BMTR[1],col22+sx-16,pos,16,sy,refreshList_tt)
				ppos=pos
				pos -=sy*2
				SGDX[0]=DS(sizeX_text,SGDX[1],col22,pos,sx,sy,SGDX[0].val,10,1024,0,sizePixelX_tt)
				pos -=sy
				SGDY[0]=DS(sizeY_text,SGDY[1],col22,pos,sx,sy,SGDY[0].val,10,1024,0,sizePixelY_tt)
				pos -=sy
				if TRGM[0].val and TUSD[0].val :
					xbut=sx/4
					TRGS[0]=DT(seed_text,TRGS[1],col22+sx-xbut,pos,xbut,sy,TRGS[0].val,useSeed_tt)
				else:xbut=0
				TRGM[0]=DT(randomOffset_text,TRGM[1],col22,pos,sx-xbut,sy,TRGM[0].val,randomOffset_tt)
				pos=ppos
				pos -=sy
			if MGSC[0].val<>2 : 
				#SGDS[0]=DS(meterPerPixel,SGDS[1],col22,pos,sx,sy,SGDS[0].val,2,10,0,mapScale_tt)
				SGDS[0]=button(SGDS,col22,pos,sx,sy,meterPerPixel,mapScale_tt)
				
		# goliath says : "systems operationnals.."
		makeBox(col1-2, py+sy*4, px+width-tx+4, py+tx-2, 0.812, 0.812, 0.812)
		text(create_text,col1,py+sy*3/2+25)
		TMBG[0]=DT(buildings_text,TMBG[1],col1,py+sy,(width-tx*3)/3,sy*3/2,TMBG[0].val,createBuildings_tt)
		TMGD[0]=DT(ground_text,TMGD[1],col1+((width-tx)/3),py+sy,(width-tx*3)/3,sy*3/2,TMGD[0].val,createGround_tt)
		cb[0]=DPB(createCity_text,cb[1],col1+2*((width-tx)/3),py+sy,(width-tx*3)/3,sy*3/2,createCity_tt)

		if sceon : bevent(no)
		
def bevent(evt):
	global MGRM,path,MLIB,escTrick,refreshConfig,init,MSET, presets,libPath ,ACATS,imageFor,none_text
	global CATS, LIBS
	if evt==cb[1] :
		errormess=""
		# check if paramaters are ok before to run
		# cases when no image loaded when source = image :
		if MBSC[0].val==1 : 
			if TLIMB[0].val==none_text : 
				errormess="you must set path & image for popmap source"
		if MGSC[0].val==1 : 
			if TLIM[0].val==none_text : 
				errormess="you must set path & image for ground source"
		# case create goround only and groundsource=none
		if TMBG[0].val==0 and TMGD[0].val==1 and MGSC[0].val==2 :
			errormess="nothing to build."
		dprint(errormess)
		if errormess=="" :
			global buildingBuild,groundBuild,popMapSourceType,groundMapSourceType,groundInfluence,buildingInfluence
			popMapSourceType=MBSC[0].val
			groundMapSourceType=MGSC[0].val
			buildingBuild=TMBG[0].val	# build building false or true
			groundBuild=TMGD[0].val		# build ground false or true
			buildingSource=MGSC[0].val	# 0 : from uv image / 1 : image from file / 2 : none (ground dims=building) / 3 : from material / 4 : last render
			groundSource=MGSC[0].val	
			groundInfluence=True
			if groundMapSourceType==2 :	# if ground source set to none,
				groundInfluence=False	# no ground influence, no ground built
				groundBuild=False
			include("primitives")
			ACATS=[]
			for id,c in enumerate(CATS) :
				if id>0 and c[1][0][0].val :
					currentcat=c[0]
					ACATS.append([[c[1][2][0].val,c[1][3][0].val,c[1][4][0].val,c[1][5][0].val,c[1][6][0].val]])
					dprint("created building library function(s) :",level=5)
					for l in c[2] :
						if l[1][0].val :
							currentlib=l[0]
							func="building_%s_%s"%(currentcat,currentlib)
							func=func.replace(" ", "_")
							ACATS[-1].append([func,l[2][0].val])
							libPath="libraries"+sys.sep+currentlib+sys.sep
							#print "building_%s_%s"%(currentcat,currentlib)
							#print libPath
							include("building","lib",libPath,cat=currentcat,funcexecname=func)
							dprint(" . %s",(func),level=5)
							#if BLIB[-1][0][6]==True :include("building","lib",libPath,"var")
			ACATS.append([[]])
			dprint("created default building library function(s) :",level=5)
			for l in CATS[0][2] :
				if l[1][0].val :
					currentlib=l[0]
					func="building_%s_%s"%("default",currentlib)
					func=func.replace(" ", "_")
					libPath="libraries"+sys.sep+currentlib+sys.sep
					ACATS[-1].append([func,l[2][0].val])
					include("building","lib",libPath,cat="default",funcexecname=func)
					dprint(" . %s",(func),level=5)
			# create city :   image dimension ,  offset with p-random on/off , build building, build ground
			createCity(max(SPMX[0].val,SPMY[0].val),[TRPM[0].val,TRPS[0].val])
		else : bell()
		
	elif evt<=10:
		if evt==1 :
			if TMn[0].val==True :
				TB[0].val=False
				TG[0].val=False
				TMp[0].val=False
			else : TMn[0].val=True
		elif evt==2 :
			if TB[0].val==True :
				TMn[0].val=False
				TG[0].val=False
				TMp[0].val=False
			else : TB[0].val=True
		elif evt==3 :
			if TG[0].val==True :
				TMn[0].val=False
				TB[0].val=False
				TMp[0].val=False
			else : TG[0].val=True
		elif evt==4 :
			if TMp[0].val==True :
				TMn[0].val=False
				TB[0].val=False
				TG[0].val=False
			else : TMp[0].val=True
	elif evt<=12: # min and max floors
		if evt==11 and nhf[0].val > nhdf[0].val:
			 nhdf[0].val=nhf[0].val
		if evt==12 and nhdf[0].val < nhf[0].val:
			 nhf[0].val=nhdf[0].val
	elif evt<=14: # min and max width
		if evt==13 and SBSMi[0].val > SBSMx[0].val:
			 SBSMx[0].val=SBSMi[0].val
		if evt==14 and SBSMx[0].val < SBSMi[0].val:
			 SBSMi[0].val=SBSMx[0].val		
	elif evt<=17: # building/ground toggles control
		if evt==16 and TMBG[0].val==False :
			if TMGD[0].val==False :TMBG[0].val=True
		if evt==17 and TMGD[0].val==False :
			if TMBG[0].val==False:TMGD[0].val=True
	
	elif evt==19: #	refresh configuration for uv image lists
		refreshConfig=[True,"map"]
	elif evt==20: #	refresh configuration for menu lists
		refreshConfig=[True,"lib"]
	elif evt<=22: # min and max floor height
		if evt==21 and SFH[0].val > SFHM[0].val :
			SFHM[0].val=SFH[0].val
		if evt==22 :
			if SFHM[0].val < SFH[0].val :SFH[0].val=SFHM[0].val
	elif evt<=25:
		if evt==23 :
			if TBLC[0].val==True :
				TBDM[0].val=False
				TBLD[0].val=False
			else : TBLC[0].val=True
		if evt==24 :
			if TBDM[0].val==True :
				TBLC[0].val=False
				TBLD[0].val=False
			else : TBDM[0].val=True
		elif evt==25 :
			if TBLD[0].val==True :
				TBLC[0].val=False
				TBDM[0].val=False
			else : TBLD[0].val=True
	elif evt<=27: # min and max interval
		if evt==26 and SISMi[0].val > SISMx[0].val:SISMx[0].val=SISMi[0].val
		if evt==27 and SISMx[0].val < SISMi[0].val:SISMi[0].val=SISMx[0].val	
	elif evt<=29: # min and max floors
		if evt==28 and SWMi[0].val > SWMx[0].val:SWMx[0].val=SWMi[0].val
		if evt==29 and SWMx[0].val < SWMi[0].val:SWMi[0].val=SWMx[0].val
	elif evt<=34 : # load save
		if evt==30 : refreshConfig=[True,"set"]			# load set
		if evt==31 :
			savePresets(presets[MSET[0].val])			# save set
			refreshConfig=[True,"refreshset"]
		elif evt==32 :									# save as set
			escTrick=2
			Blender.Window.FileSelector(savePresets	,FileSelectorSavePreset_text,path+"/presets/")
			refreshConfig=[True,"refreshset"]
		elif evt==33 :									# delete set
			erasePreset(presets[MSET[0].val])
			refreshConfig=[True,"refreshset"]
			MSET[0].val=menuIndex("presets","default")
#34 free
	elif evt<=36 : # debug
		if evt==35 :
			global debugCore
			debugCore=SDGL[0].val
		elif evt==36 :
			global debugToFile
			debugToFile=TDGO[0].val
	elif evt<=39 : # save/load config (lang and default preset)
		if evt==37 :saveConfig()
		elif evt==38 :refreshConfig=[True,"init"]
		elif evt==39 :
			global debugButtons
			debugButtons=TDBB[0].val
	elif evt<=45 : # save/load image (lang and default preset)
		if evt==40 : # load img button
			escTrick=2
			imageFor="ground"
			Blender.Window.ImageSelector(loadImage,imageSelectorLoadImage_text,path+"/libraries/common/textures/")
		elif evt==41 :loadImage(TLIM[0].val,"ground")
		if evt==42 : # load img button
			escTrick=2
			imageFor="building"
			Blender.Window.ImageSelector(loadImage,imageSelectorLoadImage_text,path+"/libraries/common/textures/")		
		elif evt==43 :loadImage(TLIMB[0].val,"building")

	elif evt==46 : refreshConfig=[True,"lang"]			# language menu
	elif evt==47 : refreshConfig=[True,"cat"]			# categories and libs
	elif evt==48 :
		refreshConfig=[True,"matmaps"]
		#materialRefresh("maps_")				# maps materials

	elif evt>=1000 and evt<=1000+len(CATS) : 
		for i in range(len(CATS)) :
			if 1000+i==evt : CATS[i][1][1][0].val=True
			else :  CATS[i][1][1][0].val=False

	elif evt>=1100 and evt<=1100+len(BLIB[thisLib][2]) : 
		for i in range(len(BLIB[thisLib][2])) :
			if 1100+i==evt : BLIB[thisLib][2][i][0][0].val=True
			else :  BLIB[thisLib][2][i][0][0].val=False
			
							#if openlib : BLIB[thisLib][2][i][0]=[D(but),1100+i,t[0],"","toggle"]
							#BLIB[thisLib][2][i][0][0]=button(BLIB[thisLib][2][i][0],tabcol,pos,tabsx,sy3)			

	Draw.Redraw(1)
			
def event(evt,val):
	global escTrick
	if evt==Draw.ESCKEY and not val and escTrick==0:Draw.Exit()

def createGround(groundList,pos,size,sizeFactor=[0,0,0],bottomFace=False):
	# building variation method (part1/2)
	# subdivide input into tiles, generates borders
	
	global mainGrid,mainCols,mainLines,mainColsOff,mainLinesOffif,groundBuild 
	xCrumb=crumb(pos[0],True) #pos[0]-int(pos[0])
	yCrumb=crumb(pos[1],True) #-int(pos[1])
	xCrumb2=crumb(pos[0]+size[0],True) #(pos[0]+size[0])-int(pos[0]+size[0])
	yCrumb2=crumb(pos[1]+size[1],True) #(pos[1]+size[1])-int(pos[1]+size[1])
	if xCrumb2==0 : xCrumb2 +=1
	if yCrumb2==0 : yCrumb2 +=1

	sqpos=[int(pos[0]),int(pos[1]),pos[2]]
	sqsize=[int(pos[0]+size[0])+crumb(pos[0]+size[0]),int(pos[1]+size[1])+crumb(pos[1]+size[1]),size[2]]
	
	for xsub in  range(sqpos[0],sqsize[0]) :
			for ysub in  range(sqpos[1],sqsize[1]) :
				borderY=borderX2=borderY2=borderX=False
				mat=[]
				if xsub==sqpos[0] :
					xPos=xCrumb
					borderX=True
				else : xPos=0
				if xsub==sqsize[0]-1:
					xPos2=xCrumb2 #=size[0]-sqsize[0]+fX-(1-(pos[0]-sqpos[0]))
					borderX2=True
				else : xPos2=1
				if ysub==sqpos[1] : 
					yPos=yCrumb #=pos[1]-sqpos[1]
					borderY=True
				else : yPos=0
				if ysub==sqsize[1]-1 :
					yPos2=yCrumb2 #=size[1]-sqsize[1]+fY-(1-(pos[1]-sqpos[1]))
					borderY2=True
				else : yPos2=1					
				faces=[borderY,borderX2,borderY2,borderX,1,0]
				#it's a border
				if faces.count(1) > 1 :
					grdpos=pos[2]
					grdborder=size[2]
					if xPos2==0 : grdpos=-10
					if TMTS[0].val and groundBuild: # tiled
						for f in faces :
							if f : mat.append(2)
						createCube(groundList,[xsub+xPos+mainColsOff,ysub+yPos+mainLinesOff,grdpos],[xPos2-xPos,yPos2-yPos,grdborder],mat,face=faces)
					if xsub>=0 and ysub>=0 and xsub<mainCols and ysub<mainLines :
						if mainGrid[xsub][ysub]<1 : mainGrid[xsub][ysub]=-1 # -1 : modifiable road 1 : protected road
				#else a tile inside
				else :
					if xsub>=0 and ysub>=0 and xsub<mainCols and ysub<mainLines :
						if mainGrid[xsub][ysub]<0 or  mainGrid[xsub][ysub]== 3 : mainGrid[xsub][ysub]=0 # 0 modifiable sidewalk 4 protected
	if groundBuild:
		pos[0] +=mainColsOff
		pos[1] +=mainLinesOff
		createCube(groundList,pos,size,[2,2,2,2,2])

# PSEUDO or TRUE RANDOMS FUNCTIONS
def Prandom(x="dep",y="dep") :
	if TUSD[0].val== True :
		global pseudoInc
		pseudoInc +=1
		Blender.Noise.setRandomSeed(SSED[0].val*pseudoInc)
		return Blender.Noise.random()
	else :
		return random()
		
def Prandint(a,b,x="dep",y="dep") :
	a=int(a)
	b=int(b)
	if TUSD[0].val== True :
		global pseudoInc
		pseudoInc +=1
		Blender.Noise.setRandomSeed(SSED[0].val*pseudoInc)
		iv=max(a,b)-min(a,b)+1
		return int(min(a,b))+int(Blender.Noise.random()*iv)
	else :
		return randint(a,b)

def Puniform(a,b,x="dep",y="dep") :
	if TUSD[0].val== True :
		global pseudoInc
		pseudoInc +=1
		Blender.Noise.setRandomSeed(int(SSED[0].val)*pseudoInc)
		return min(a,b)+Blender.Noise.random()*(max(a,b)-min(a,b))
	else :
		return uniform(min(a,b),max(a,b))

		
# #################################################################################################### city code
# to generate cities, we need a population density map,
# next we place cubes and size them vertically according to the population density
def createCityMesh():
	dprint("<END>")
	# function variables
	global libPath, debugCore, timeStart, mainGrid,mainCols,mainLines,mainColsOff,mainLinesOff, pseudoInc, groundBuild,pBar
	global ACATS,pseudoInc,popMap,groundMap
	global buildingBuild,groundBuild,buildingSource,groundSource,groundInfluence,buildingInfluence
	pBar +=0.015
	Window.DrawProgressBar(pBar,"init...")	

	# vertices, faces, smooth, mats
	cityList=[[],[],[],[]]
	groundList=[[],[],[],[]]
	buildingList=[]
	ps=1.0 # pixel size: one pixel in the pop map is that big in blender units
	bsz=[1.,1.,1.] # x,y,z standard building size
	pos=[0,0,0]
	swpos=[0,0,0]
	rndbh=0
	strech=[[0,0],[0,0],[0,0],[0,0]]
	failed="" # lib. error control

	polyground=True
	
	# # function variables from gui
	# what to build :
	"""
	buildingBuild=TMBG[0].val	# build building false or true
	groundBuild=TMGD[0].val		# build ground false or true
	buildingSource=MGSC[0].val	# 0 : from uv image / 1 : image from file / 2 : none (ground dims=building) / 3 : from material / 4 : last render
	groundSource=MGSC[0].val	
	groundInfluence=True
	if groundSource==2 :		# if ground source set to none,
		groundInfluence=False	# no ground influence, no ground built
		groundBuild=False
	"""
	#elif buildingBuild and groundBuild : groundInfluence=True
	#elif groundBuild : 
	#else : groundInfluence=TRGP[0].val

	#buildingInfluence=TRPP[0].val	
	if groundInfluence==False and buildingInfluence==False :sidewalkExist=swk=False
	else : sidewalkExist=swk=TMSW[0].val
	if sidewalkExist:swStich=TSWS[0].val		# stich sidewalks to roads
	else : swStich=False
	
	# buildings
	floorMin=SFH[0].val			# floor min height
	floorMax=SFHM[0].val		# floor max height
	fHeightInt=[SFH[0].val,SFHM[0].val]
	ffHeightVar=SFFH[0].val
	minbh=nhf[0].val			# max floor nb
	maxbh=nhdf[0].val			# min floor nb
	bHeightInt=[nhf[0].val,nhdf[0].val]
	varbh=SBHV[0].val			# floor nb var
	bSizeInt=[SBSMi[0].val,SBSMx[0].val]							# min-max building width
	iSizeInt=[SISMi[0].val,SISMx[0].val]							# min-max space interval between building
	aSizeIntX=[bSizeInt[0]+iSizeInt[0]*2,bSizeInt[1]+iSizeInt[1]*2]	# min-max building spawning areas
	aSizeIntY=aSizeIntX												# if sidewalks, it's re-computed at each spawn.
	scatter=1-SCAT[0].val		# scattering
	
	threshold=STRT[0].val		# this is the color under which no buildings will be created
	bhcoef=1/(1-threshold)
	#rlf=SRBL[0].val
	checkRatioHV=THWR[0].val	# HWratio control
	ratioHV=SHWR[0].val			# ration between height and width
	lod=SMDB[0].val				# building LOD

	#Bblend=SBBV[0].val			# proba to blend buildings
	#BblendNb=SBBVnb[0].val				# proba to blend buildings
	
	# ground
	tiled=TMTS[0].val
	swSizeInt=[SWMi[0].val,SWMx[0].val]	# sidewalks min width unit
	swh=SWH[0].val       	 	# sidewalks height
	blkStich=TBGS[0].val		# stich buildings to roads
	blkToGrid=TBGG[0].val
	swmethod=SWMM[0].val		# block or building method prob. 100=building, 0=block, 50 half/half
	swmethod2=SWSR[0].val		# stich to pixel units prob.

	if tiled==False :
		swVsmooth=SWSV[0].val	# sidewalks vertical bevel quality
		swHsmooth=SWSH[0].val	# sidewalks horizontal bevel quality
		swHbvl=SWB[0].val  		# sidewalks corner horizontal bevel size
		swb=swh*1.5				# sidewalks border width

		# create vertical bevel vector list
		if swVsmooth>0 :
			swvb=(float(swh)/3)*2
			swhb=(float(swb)/3)*2
			swbevel=[[[0,0,0],[0,swvb,0],[swhb,0,0],[0,-swvb,0],[-swhb,0,0]]]
			size=swb/3
			swbevel=polyCorners(swbevel,size,swVsmooth,type="vector")
			swbevel=swbevel[0][0:swVsmooth+1]
			swbevel.append([swb/3,0,0])
		else :swbevel=[[0,swh,0]]
		
	# for special cases during the spawn loop :
	if sidewalkExist and swSizeInt[0] > iSizeInt[0] :expandArea=True
	else : expandArea=False

	swsz=[0,0,swh]

	if swk:zpos=swh
	else:zpos=0


	
	camOb=Object.Get("Camera")
	cityOb=Object.Get("city")
	# ############ MAPS STUFF

	citySZ=SGDS[0].val			# meter per pixel for streetsmap and readStreets
	blkSZ=SBD[0].val			# average space between each group of buiding origin (.25*maxbh)/ratioHV
	#ratioSZ=int(blkSZ/citySZ)	# equals the nb of street units in one block unit
	# popmap(=popMap) size in pixels
	#if buildingSource==3 :
	#	blkCols=SPMX[0].val
	#	blkLines=SPMY[0].val
	#else :
	blkCols=popMap.size[0]
	blkLines=popMap.size[1]	
		
	if groundInfluence :
		if groundMapSourceType==3 :
			cityCols=SGDX[0].val
			cityLines=SGDY[0].val
		else :
			cityCols=groundMap.size[0]
			cityLines=groundMap.size[1]
	else :
		cityCols=0
		cityLines=0

	# resulting global map size in meters 
	mainCols=max(cityCols*citySZ+2,blkCols*blkSZ)
	mainLines=max(cityLines*citySZ+2,blkLines*blkSZ)
	
	# map offsets calculation in meters
	mainColsOff=-int(mainCols/2) #(mainCols/citySZ/2)-(cityCols/2)
	mainLinesOff=-int(mainLines/2) #(mainLines/citySZ/2)-(cityLines/2)

	#cityColsOff=-int(cityCols*citySZ/2) # (mainCols/citySZ/2)-(blkCols*blkSZ/citySZ/2)
	#cityLinesOff=-int(cityLines*citySZ/2)
	cityColsOffGrid=int((mainCols-(cityCols*citySZ))/2)
	cityLinesOffGrid=int((mainLines-(cityLines*citySZ))/2)
	blkColsOffGrid=int((mainCols-(blkCols*blkSZ))/2)
	blkLinesOffGrid=int((mainLines-(blkLines*blkSZ))/2)
	#MapColsOff=blkColsOff-mainColsOff
	#MapLinesOff=blkLinesOff-mainLinesOff
	pBar +=0.030
	Window.DrawProgressBar(pBar,"init...")		
	# building elevation map scaled - fill the block grid with popmap heights. -1 if texture color<threshold,  else float=elevation factor
	#	-1	no elevation
	#	>=0	elevation units
	blkGrid= [[-1 for col in range(mainLines)] for row in range(mainCols)]
	for x in range(blkCols) :
		for y in range(blkLines) :
			if popMap.getPixelF(x,y)[0] >=threshold :
				for xMain in range(x*blkSZ,x*blkSZ+blkSZ) :
					for yMain in range(y*blkSZ,y*blkSZ+blkSZ) :
						blkGrid[xMain+blkColsOffGrid][yMain+blkLinesOffGrid]=popMap.getPixelF(x,y)[0]
	# fill the main grid with streets reservation from grd map. 
	# 	1	road (from groundmap) 
	# 	0	sidewalks 
	# 	-1	can become a sidewalk or a road depending on the algo 
	#	2	building here or blkGrid < threshold here
	if groundInfluence : fill=0
	else : fill = -1
	fill=0
	mainGrid=[[fill for col in range(mainLines)] for row in range(mainCols)]
	zero=tot=mainLines*mainCols
	one=0
	two=0
	pBar +=0.030
	Window.DrawProgressBar(pBar,"init...")
	if groundInfluence :
		for x in range(cityCols) :
			for y in range(cityLines) :
				#col=groundMap.getPixelF(x,y)
				#if col[3]<>1 : print x,y
				#groundMap.setPixelF(x,y,col)
				if groundMap.getPixelF(x,y)[0] <= 0.1 : #or groundMap.getPixelF(x,y)[3] < 1 :
					for xMain in range(x*citySZ,x*citySZ+citySZ) :
						for yMain in range(y*citySZ,y*citySZ+citySZ) :
							mainGrid[xMain+cityColsOffGrid][yMain+cityLinesOffGrid]=1
							one +=1
							zero -=1
		# add a black frame (poly parser helper)
		for x in range(cityCols*citySZ) :
			mainGrid[cityColsOffGrid+x][cityLinesOffGrid-1]=1
			mainGrid[cityColsOffGrid+x][cityLinesOffGrid+cityLines*citySZ]=1
		for y in range(cityLines*citySZ+2) :
			mainGrid[cityColsOffGrid-1][cityLinesOffGrid-1+y]=1
			mainGrid[cityColsOffGrid+cityCols*citySZ][cityLinesOffGrid-1+y]=1
				
	for x in range(mainCols) :
		for y in range(mainLines) :
			if blkGrid[x][y] == -1 and mainGrid[x][y] <> 1:
				mainGrid[x][y]=2
				two +=1
				zero -=1

	# for debug
	dprint("final city size : %sx%s m.",(mainCols,mainLines),level=4)
	#print "StreetMap Offsets : X %s Y %s groundMap units" %(mainColsOff,mainLinesOff)
	dprint("building popMap : %sx%s px. unit:%s m. ",(blkCols, blkLines, blkSZ),level=4)
	dprint("groundMap : %sx%s px. unit:%s m",(cityCols, cityLines, citySZ),level=4)
	dprint("0 : %s - 1 : %s - 2 : %s ",((zero*100)/tot,(one*100)/tot,(two*100)/tot),level=4)
	#print "popMap Offsets : X from %s to %s - Y from %s to %s" %(blkColsFirst,blkColsLast,blkLinesFirst,blkLinesLast)
	pBar +=0.015
	Window.DrawProgressBar(pBar,"init...")
	
	#COLOR/BUILDING HEIGHT CALIBRATION
	minCol=1
	maxCol=0
	for x in range(blkCols):
		for y in range(blkLines):
			pxl=popMap.getPixelF(x,y)
			if pxl[0] >= threshold:
				minCol=min(pxl[0],minCol)
				maxCol=max(pxl[0],maxCol)
	intCol=maxCol-minCol
	if intCol==0 : 
		intCol=1#max(0.1,maxCol-threshold)
		minCol=threshold
	intBh=bHeightInt[1]-bHeightInt[0]
	
	
	# CREATE export png MAP
	future=False
	near=True
	if future==near :
		pass
		# wip : generate a scaled map from ground map and pop map :
		# buildings dim loc and size are freezed and scaled to ground scale. easier export, easier editing with gimp etc..
		# need another way to parse elevations : one popMap pixel is not a (group of) building but a part of a building
		# need to parse ground an pop data from the same map in a different way.
		# 
		
		# define city dimension
		#mainCols=max(cityCols,blkCols*ratioSZ) *** variable already used, but in meter not in grd
		#mainLines=max(cityLines,blkLines*ratioSZ)
		#ctySZ=min(blkSZ,citySZ)
		#create map
		#try :
		#	readStreets=Image.Get("readStreets")
		#	readStreets.setName("dead")
		#except :
		#	readStreets=Image.New("readStreets",mainCols,mainLines,32)
		# add popMap
		#for x in range(blkCols):
		#	for y in range(blkLines):
		#		pxl=popMap.getPixelF(x,y)
		#		if pxl[0] > threshold:
		#			for xsub in range(int(x*ratioSZ),int((x+1)*ratioSZ)) :
		#				for ysub in range(int(y*ratioSZ),int((y+1)*ratioSZ)) :
		#					#print "pop %s %s" %(xsub,ysub)
		#					readStreets.setPixelF(xsub, ysub, (pxl[0], pxl[1], pxl[2], pxl[3]))
		# add groundMap
		#for x in range(cityCols):
		#	for y in range(cityLines):
		#		pxl=groundMap.getPixelF(x,y)
		#		if pxl[0] < threshold:
		#				readStreets.setPixelF(x, y, (pxl[0], pxl[1], pxl[2], pxl[3]))

		#print "city map dim : %s x %s (%s meters square)" %(mainCols,mainLines,mainCols*citySZ)	
	#stats	
	totPixels=blkCols*blkLines*1.0
	totBuildings=0			
	# ############ init loop var

	x=yfirst=x2=y2=0
	pos[1]=0
	bsz[1]=0
	InitLine=True
	spawn=True # as long as there's some place for a building, spawn them
	xpast=-1
	xpast2=-1
	ypast=-1
	ypast2=-1
	bug=0
	uv=0
	#lod=0
	spawnIt=0
	ypBar=0
	# ########################################################################### MAIN LOOP
	pBar=0.1
	if groundBuild and buildingBuild : pBarCeil=0.1
	else : pBarCeil=0.3

	dprint("+ LOCATION ALGO",level=2)
	drop=False
	
	if buildingBuild or buildingInfluence :
		timeSpawn=sys.time()
		while spawn :
			# main loop control 
			#y = nearInt(pos[1]+bsz[1])
			if sys.time()-timeSpawn>10 or yfirst>ypBar :
				Window.DrawProgressBar(pBar+(yfirst*pBarCeil)/mainLines,"Buildings location/volume...")
				ypBar=yfirst
				if sys.time()-timeSpawn>10 :
					ypBar=0
					timeSpawn=sys.time()
			y = int(pos[0]+bsz[0])+crumb(pos[0]+bsz[0])+1
			if drop:y=yfirst
			# back to first line ?
			if y + aSizeIntY[0] >= mainLines :
				y=0
				InitLine=True
				#print "-----------------------------------------------------"
			# look for an area. starting from line y, scan all the row for a free place for origin
			#then look if at least a min size building can be built here
			x=0
			spawnIt +=1
			noArea=True
			dprint(" + origin %s %s",(x,y),level=2)
			while noArea :
				#if mainGrid[x][y]<=0 : status=True else : 
				yfirst=y
				x,status=move(mainGrid,[x,y],mainCols - aSizeIntX[0],'x',False)
				#if status==False:print "false"
				if status :
					y,status=move(mainGrid,[x,y],-mainLines,'y',True)
					x,status=move(mainGrid,[x,y],-mainCols,'x',True)
				if status :
					for y2 in range(y,y+int(aSizeIntY[0])) :
						if mainGrid[x][y2]>=1 :
							status=False
							y2 -=1
							x2=x
							break
						else :
							x2,status=move(mainGrid,[x,y2],aSizeIntX[0]-1,'x',True)
							if x2-x<aSizeIntX[0]-1 :
								status=False
								break
					if status: noArea=False
					else :
						for yfill in range(y,y2+1) :
							for xfill in range(x,x2+1) :
								if xfill<mainCols and yfill<mainLines and mainGrid[xfill][yfill]<>1 : mainGrid[xfill][yfill]=3
						#createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-40),(xfill-x+1,yfill-y+1,10),1,lod,orientation)
				if status==False :
					if y<yfirst:y=yfirst
					else:y=yfirst+1
					x=0
					if y + aSizeIntY[0] >= mainLines :
						if InitLine :
							spawn=False
							noArea=False
						else :
							y=0
							InitLine=True
							
			InitLine=False
			
			#if debugCore :
			#	if x==xpast and y==ypast and x2==xpast2 and y2==ypast2 :
			#		bug +=1
			#		if bug==2:spawn = False
			#	xpast=x
			#	xpast2=x2
			#	ypast=y
			#	ypast2=y2
			dprint(" + starting from min area x from pos %s to %s and y from pos %s to %s",(x,x2,y,y2),level=2)
			# must not append
			#if mainGrid[x][y]>=1 :
			#	dprint("offset bug. exit")
			#	raise() 
			#if mainGrid[x][y+1]>=1 :
			#	dprint("no comprendo. exit")
			#	raise()

			# LOOP COMPLETED
			if spawn == False :
				dprint("+ DONE. ended @ area x from pos %s to %s and y from pos %s to %s",(x,x2,y,y2),level=2)
				continue

			#try to expand the min area we've just found
			x3=mainCols+1
			y3=mainLines+1
			for y2 in range(y,y+int(aSizeIntY[0])) :
				x2,status=move(mainGrid,[x,y2],aSizeIntX[1]-1 ,'x',True)
				x3=min(x2,x3)
			for x2 in range(x,x+int(aSizeIntX[0])) :
				y2,status=move(mainGrid,[x2,y],aSizeIntY[1]-1 ,'y',True)
				y3=min(y2,y3)	
			if x3-x >= y3-y :
				for x2 in range(x,x3+1) :
					y2,status=move(mainGrid,[x2,y],aSizeIntY[1]-1 ,'y',True)
					y3=min(y2,y3)
			else :
				for y2 in range(y,y3+1) :
					x2,status=move(mainGrid,[x,y2],aSizeIntX[1]-1 ,'x',True)
					x3=min(x2,x3)	
			#print "max area x from pos %s to %s and y from pos %s to %s" %(x-mainColsOff,x2-mainColsOff,y-mainLinesOff,y2-mainLinesOff)
			#if x>=165 and y>220: spawn=False
			aX=x3-x+1
			aY=y3-y+1


			
			#xRand=Prandint(iSizeInt[0],iSizeInt[1],pos[2]+bsz[1]*pos[0],pos[1]+bsz[1]*pos[0])
			#yRand=Prandint(iSizeInt[0],iSizeInt[1],pos[1]+bsz[1]*pos[2],pos[1]+bsz[0]*pos[0])			

			if blkGrid[x][y]>=0 :
			
				redPercent=blkGrid[x][y]
				# this subdivide a building into many
				buildingsPerPixel=1 #randint(1,max(1,mbpp)) #+int(math.floor(1/bsz[2]))))
				#streetmod :
				xn=yn=0
				if swk:
					swx1=swy1=swx2=swy2=-1
					swrnd=Prandint(0,100,x,y) # for around building or around block methods

				for building in range(int(buildingsPerPixel)):
					brk=False
					build=True
					drop=False
					xOff=yOff=0	# building loc offset because of streets
					onStr=True	# true if the street map is below the popMap
					resized=False
					orientation=[]
				# #################################
				# building size, height and position in the determined area
				# #################################
				
				# nearest roads
					# If needed, look if there's streets around the area
					# reminder : worst case possible when sw sizes are greater then int sizes : if sidewalks on and roads found there's some case we have to try to expand again the area.
					# if it failed, we'll have to shorten theoffsets/the building sizes, or cancel if too small.
					streetW=streetS=streetE=streetN=0
					expandableN=expandableE=False
					expandN=expandE=0
					if sidewalkExist or blkStich :
						if x+aX<mainCols :
							for s in range(y+1,y+aY-1) :
								if mainGrid[x+aX][s] == 1 : streetE +=1
							if streetE==0 : expandableE=True
						if y+aY<mainLines :
							for s in range(x,x+aX) :
								if mainGrid[s][y+aY] == 1 : streetN +=1
							if streetN==0 : expandableN=True							
						if x>0 :
							for s in range(y,y+aY) :
								if mainGrid[x-1][s] == 1 : streetW +=1
							if streetW>0 and expandArea and expandableE and swSizeInt[1]>iSizeInt[1]:
								for s in range(x+aX+1,x+aX+1+int((swSizeInt[1]-iSizeInt[1])*2)) :
									if s < mainCols :
										y2,status=move([s,y],aY,'y',True)
										if y2==y+aY : expandE +=1
										else : s=mainCols
								dprint("  expanded East Area : %s",(expandE),level=2)
						if y>0 :
							for s in range(x,x+aX) :
								if mainGrid[s][y-1] == 1 : streetS +=1
							if streetS>0 and expandArea and expandableN and swSizeInt[1]>iSizeInt[1]:
								for s in range(y+aY+1,y+aY+1+int((swSizeInt[1]-iSizeInt[1])*2)) :
									if s < mainLines :
										x2,status=move([x,s],aX,'x',True)
										if x2==x+aX : expandN +=1
										else : s=mainLines
									dprint("  expanded North Area : %s",(expandN),level=2)

						aX +=expandE
						aY +=expandN

				# location minimum X and Y offset in the area
					#  minimum offsets are :
					xOff=yOff=0
					if streetW+streetS+streetE+streetN > 0 and sidewalkExist :
						if streetW > 0 : xOff=swSizeInt[0]		# align West
						else : xOff=iSizeInt[0]
						if streetE > 0 : xOff +=swSizeInt[0]	# align East
						else : xOff +=iSizeInt[0]
						if streetS > 0 : yOff=swSizeInt[0]		# align south
						else : yOff=iSizeInt[0]
						if streetN > 0 : yOff +=swSizeInt[0]	# align north			
						else : yOff +=iSizeInt[0]
					else :xOff=yOff=iSizeInt[0]*2
					
				# Height
					# desired Building Height from map :
					# change size of building based on pixel red color
					bSize=bHeightInt[0]+int(((redPercent-minCol)*intBh)/intCol)		# desired height (calibration)
					# randomize number of floors
					if varbh>0 : bSize=Prandint(min(bHeightInt[0],bSize-(bSize*varbh)),max(bHeightInt[1]+4,bSize+(bSize*varbh)),x*bSize,y*building)
					floor=Puniform(fHeightInt[0],fHeightInt[1])					# randomize floor height
					ffloor=Puniform(floor,floor*ffHeightVar)
					bsz[2]=ffloor+floor*(bSize-1)								# final building height
					
				# building sizes
					# to high : make it shorter to fit in the available area, with minimum offsets
					# min and max size constrained by height and area offsets
					if checkRatioHV and min(aX-xOff,aY-yOff) < bsz[2]/ratioHV :
						bsz[2]=ffloor+(int((min(aX-xOff,aY-yOff)*ratioHV)/floor)-1)*floor
						minSizeFactor=max(bSizeInt[0],bsz[2]/ratioHV)
					else : minSizeFactor=bSizeInt[0]
					maxSizeFactorX=min(bSizeInt[1],aX-xOff-expandE) # max building widths constrained by available area
					maxSizeFactorY=min(bSizeInt[1],aY-yOff-expandN)
					bsz[0]=Puniform(minSizeFactor,maxSizeFactorX) #*randint(1,coefbs)
					bsz[1]=Puniform(minSizeFactor,maxSizeFactorY) #*randint(1,coefbs)

				# location
					# determine absolute offsets
					maxIntX=min(aX-bsz[0]-iSizeInt[0],iSizeInt[1])
					maxIntY=min(aY-bsz[1]-iSizeInt[0],iSizeInt[1])
					xAbsIntOff=Puniform(iSizeInt[0],maxIntX)
					yAbsIntOff=Puniform(iSizeInt[0],maxIntY)
					# sidewalks are enabled
					if sidewalkExist :
						if aX-bsz[0]<swSizeInt[0]*2 and streetE+streetW > 0 :
							#if streetE<>0 and streetW <>0 :
							bsz[0]=aX-swSizeInt[0]*2
							#else : bsz[0]=aX-swSizeInt[0]
							dprint ("worst case : building x size does not fit with sidewalks",level=2)
							if bsz[0]<bSizeInt[0] :
								dprint("  x to small : drop",level=2)
								bsz[0]=bSizeInt[0]
								drop=True
	 
						maxIntX=min(aX-bsz[0]-swSizeInt[0],swSizeInt[1])
						xAbsSwOff=Puniform(swSizeInt[0],maxIntX)
						xAbsRoadOff=xAbsSwOff
						
						if aY-bsz[1]<swSizeInt[0]*2 and streetS+streetN > 0 :
							#if streetS<>0 and streetN <>0 :bsz[1]=aY-swSizeInt[0]*2
							#else : bsz[1]=aY-swSizeInt[0]
							bsz[1]=int(aY-swSizeInt[0]*2)
							dprint ("worst case : building y size smaller",level=2)
							if bsz[1]<bSizeInt[0] :
								dprint("  y to small : drop",level=2)
								bsz[1]=bSizeInt[0]
								drop=True
								
						maxIntY=min(aY-bsz[1]-swSizeInt[0],swSizeInt[1])
						yAbsSwOff=Puniform(swSizeInt[0],maxIntY)
						yAbsRoadOff=yAbsSwOff

						if blkStich==False :
							if aX-bsz[0]-xAbsRoadOff-swSizeInt[0]>0:
								xAbsRoadOff=Puniform(xAbsRoadOff,xAbsRoadOff+aX-bsz[0]-xAbsRoadOff-swSizeInt[0])
							if aY-bsz[1]-yAbsRoadOff-swSizeInt[0]>0:
								yAbsRoadOff=Puniform(yAbsRoadOff,yAbsRoadOff+aY-bsz[1]-yAbsRoadOff-swSizeInt[0])					
						
						# re-height in worst case
						if checkRatioHV and streetW+streetS+streetE+streetN > 0 and bsz[2]/ratioHV > min(bsz[0],bsz[1]) :
								bsz[2]=ffloor+(int((min(bsz[0],bsz[1])*ratioHV)/floor)-1)*floor
					# else normal interval as sidewalks
					else :
						xAbsRoadOff=xAbsIntOff
						yAbsRoadOff=yAbsIntOff
						
					#  if needed, location offsets are calculated from roads
					if streetW+streetS+streetE+streetN > 0 and (blkStich or sidewalkExist):
						if streetW > 0: # align west
							xOff=xAbsRoadOff
							dprint(" .building oriented west",level=2)
						elif streetE > 0: # align East
							xOff=aX-bsz[0]-xAbsRoadOff
							dprint(" .building oriented east",level=2)
						else : xOff=xAbsIntOff
						if streetS > 0:# align south
							yOff=yAbsRoadOff
							dprint(" .building oriented south",level=2)
						elif streetN > 0:# align north			
							yOff=aY-bsz[1]-yAbsRoadOff
							dprint(" .building oriented north",level=2)
						else : yOff=yAbsIntOff
					# no roads around, or no need. location offsets from SW
					else :
						xOff=xAbsIntOff
						yOff=yAbsIntOff
				#not sure it's useful anymore.
					#if rlf :
					#	rlfxmax=x2-x+1-bsz[0]-iSizeInt[0]*2
					#	rlfymax=y2-y+1-bsz[1]-iSizeInt[0]*2
					#	xRand=Puniform(iSizeInt[0],rlfxmax)   #blkStich
					#	yRand=Puniform(iSizeInt[0],rlfymax)   #blkStich
					#else : xRand=yRand=0
					xRand=yRand=0
					
				# Building Position
					pos=[x + xRand + xOff , y + yRand + yOff  ,zpos]					

				# write building allocated space to maingrid
					# convert coords to int for array
					#intPos0=nearInt(pos[0])
					#intPos1=nearInt(pos[1])
					#intPosBsz0=nearInt(intPos0+bsz[0])
					#intPosBsz1=nearInt(intPos1+bsz[1])
					intPosBsz=[int(pos[0]+bsz[0])+crumb(pos[0]+bsz[0]),int(pos[1]+bsz[1])+crumb(pos[1]+bsz[1])]
					intPos=[int(pos[0]),int(pos[1])]


					# calculation Area W and S :
					for xfill in range(x+(int(pos[0])-x)/2,intPosBsz[0]) :
						for yfill in range(y+(int(pos[1])-y)/2,intPosBsz[1]) :
							if xfill<mainCols and yfill<mainLines and mainGrid[xfill][yfill]<>1 : mainGrid[xfill][yfill]=3
					#createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-30),(intPosBsz0-x,intPosBsz1-y,2),1,1,lod,orientation)
					#createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-30),(aX,aY,2),1,1,lod,orientation)

					# building itself
					for xfill in range(intPos[0],intPosBsz[0]) :
						for yfill in range(intPos[1],intPosBsz[1]) :
							if xfill<mainCols and yfill<mainLines and mainGrid[xfill][yfill]<>1 : mainGrid[xfill][yfill]=2
					
					# discard some buildings and far away buildings (more scattered)
					if scatter<>1 and Prandom() < fabs(pos[0]-mainCols/2.0)/(mainCols/2.0)-scatter or Prandom() < fabs(pos[1]-mainLines/2.0)/(mainLines/2.0)-scatter:
						drop=True
						
					
					if blkToGrid :
						bsz[0]=intPosBsz[0]-intPos[0]
						bsz[1]=intPosBsz[1]-intPos[1]
						pos[0]=intPos[0]
						pos[1]=intPos[1]

					# append to building list
					#if drop : pos[2]=-300
					if buildingBuild and drop==False :
						totBuildings+=1
						orientation=[streetS,streetE,streetN,streetW]
						# unused
						uv=0
						buildingList.append([pos,[bsz[0],bsz[1],bsz[2] ],ffloor,floor,lod,orientation])


					# new building changes terrain props :
					if buildingInfluence :
						# sidewalks 
						if sidewalkExist :
							xSwOff=xSwOff2=ySwOff=ySwOff2=0
							if streetW+streetS+streetE+streetN > 0 :
								if streetW > 0: # align west
									if swStich : xSwOff=xAbsRoadOff
									else : xSwOff=Puniform(swSizeInt[0],min(xAbsRoadOff,swSizeInt[1]))
								elif streetE > 0: # align East
									if swStich : xSwOff2=xAbsRoadOff
									else : xSwOff2=Puniform(swSizeInt[0],min(xAbsRoadOff,swSizeInt[1]))
								if streetS > 0:# align south
									if swStich : ySwOff=yAbsRoadOff
									else : ySwOff=Puniform(swSizeInt[0],min(yAbsRoadOff,swSizeInt[1]))
								elif streetN > 0:# align north			
									if swStich : ySwOff2=yAbsRoadOff
									else : ySwOff2=Puniform(swSizeInt[0],min(yAbsRoadOff,swSizeInt[1]))

							if xSwOff==0:	xSwOff=Puniform(swSizeInt[0],swSizeInt[1])
							if xSwOff2==0:	xSwOff2=Puniform(swSizeInt[0],min(aX-bsz[0]-xSwOff,swSizeInt[1]))
							if ySwOff==0:	ySwOff=Puniform(swSizeInt[0],swSizeInt[1])
							if ySwOff2==0:	ySwOff2=Puniform(swSizeInt[0],min(aY-bsz[1]-ySwOff,swSizeInt[1]))
							#
							swsz[0]=min(aX,bsz[0]+xSwOff+xSwOff2)
							swsz[1]=min(aY,bsz[1]+ySwOff+ySwOff2)
							swsz[2]=swh
							swpos[0]=max(x,pos[0]-xSwOff)
							swpos[1]=max(y,pos[1]-ySwOff)
							swpos[2]=0
							createGround(groundList,swpos,swsz)


			else: 
				dprint("no building here",level=2)
				dprint(blkGrid[x][y],level=2)
			# error control : librarie or primitives bugged : abort	
			if failed<>"" :
				#brk=True
				break



		dprint("%s buildings and sidewalks located in %.2f seconds",(totBuildings, sys.time()-timeStart))

	
	# Build Ground
	# #########################
	# (some ground parts have perhaps been built, if buildings loc has influence over the ground)
	timeGround=sys.time()
	polyMainGrid=[]
	for coord in mainGrid :polyMainGrid.append(coord[:])
	for x in range(mainCols):
		for y in range(mainLines):
			if mainGrid[x][y]==3 : # used by building pos algo
				if buildingInfluence : mainGrid[x][y]=1
				else : mainGrid[x][y]=0
			elif mainGrid[x][y]==-1 :
				mainGrid[x][y]=1
			elif mainGrid[x][y]==0 and sidewalkExist==False :
				mainGrid[x][y]=1
				
			if mainGrid[x][y]==-1 : polyMainGrid[x][y]=1
			elif polyMainGrid[x][y]<>1:  polyMainGrid[x][y]=0
	dprint("main Ground re-allocated in %.2f seconds",(sys.time()-timeGround))

	# build
	if groundBuild and failed=="" :
		if buildingBuild : pBar=0.3
		totTiles=(cityCols*cityLines)*citySZ**2
		Window.DrawProgressBar(pBar,"Ground...")
		# build simple ground if no tiles, whatever the other options are
		if tiled == False : 
			#createPlane([vs_grd,fs_grd,sm_grd,mat_grd],[mainColsOff,mainLinesOff,0],[mainCols,mainLines,0],[0])
			createPlane(groundList,[mainColsOff,mainLinesOff,0],[mainCols,mainLines,0],[0])
		# if no sidewalks ground will be flat : swkpos=-1
		if swk==False : swh +=0.01

		if tiled  :
		#if tiled or groundInfluence :
			nx=mainCols/10
			for x in range(mainCols):
				if x>nx :
					Window.DrawProgressBar(pBar+(x*pBarCeil)/mainCols,"Ground (tiled)...")
					nx=(mainCols/10)*int(x*10/mainCols)+1
				for y in range(mainLines):
					#if mainGrid[x][y]==3 : # used by building pos algo
					#	if buildingInfluence : mainGrid[x][y]=1
					#	else : mainGrid[x][y]=0
					faces=[0,0,0,0,1,0]
					mats=[]
					if sidewalkExist :
						borderX=borderX2=borderY=borderY2=False
						if mainGrid[x][y]==0 or mainGrid[x][y]==4 or mainGrid[x][y]==2 : # a part of sidewalk (0) perhaps something on it (2=building or interbuilding)
							if x>0 and mainGrid[x-1][y]==1 : borderX=True
							if x<mainCols-1 and mainGrid[x+1][y]==1 : borderX2=True
							if y>0 and mainGrid[x][y-1]==1 : borderY=True
							if y<mainLines-1 and mainGrid[x][y+1]==1 : borderY2=True
							faces=[borderY,borderX2,borderY2,borderX,1,0]
							for f in faces :
								if f : mats.append(2)
							#createCube([vs_grd,fs_grd,sm_grd,mat_grd],[x+mainColsOff,y+mainLinesOff,0],[1,1,swh],mats,face=faces)
							createCube(groundList,[x+mainColsOff,y+mainLinesOff,0],[1,1,swh],mats,face=faces)
					else : mainGrid[x][y]=1 # if no sidewalks, convert to road
					if mainGrid[x][y]==1 or mainGrid[x][y]==-1  : # a part of road (1) or unchanged space (-1) or algo buildpos mark (3)
						#createPlane([vs_grd,fs_grd,sm_grd,mat_grd],[x+mainColsOff,y+mainLinesOff,0],[1,1,0],[1])
						createPlane(groundList,[x+mainColsOff,y+mainLinesOff,0],[1,1,0],[1])
			dprint("main Ground (%s tiles) created in %.2f seconds",(totTiles,sys.time()-timeGround))
		else :
			# ground as polygons
			pdebug=False
			# debug : test map in uv to see the image parser in action
			if pdebug :
				try :test=Image.Get("test")
				except :test=Image.New("test",mainCols,mainLines,32)
				verystartcol=(1,0,0,1)
				finishcol=(0,1,1,1)
				fill=(1,1,1,1)
				current=(0.7,.7,.7,1)
				blue=(0,0,1,1)
				notok=(0,1,0,1)
				testin=(0,0,1,1)
				last=0
				bpy.data.images.active = test
				for x in range(mainCols) :
					for y in range(mainLines) :
						if polyMainGrid[x][y] == 0 :
							test.setPixelF(x, y, blue)
			
			# /debug
			
			xo=yo=x=0
			y=cityLinesOffGrid-1
			groundPolys=[]
			groundPos=[]
			pxpoly=0
			pxdone=5
			dir=[[1,0],[0,1],[-1,0],[0,-1]]


			if mainCols==cityCols*citySZ+2 :
				xfirst=0
				xlast=mainCols
			else :
				xfirst=cityColsOffGrid-1
				xlast=xfirst+cityCols*citySZ+2
			if mainLines==cityLines*citySZ+2 :
				yfirst=0
				ylast=mainLines
			else :
				yfirst=cityLinesOffGrid-1
				ylast=yfirst+cityLines*citySZ+2
			
			ny=ylast/10				
			while y < ylast-1:
				x=xfirst
				y +=1
				if y>ny :
					Window.DrawProgressBar(pBar+(y*pBarCeil)/ylast,"Ground (gons)...")
					ny=(ylast/10)*int(y*10/ylast)+1
				while x < xlast :
					#print x,y
					#white=0
					#if x < xlast-1 :
						#if polyMainGrid[x+1][y]==0 : white==1
					if pdebug : print "search %s %s..."%(x,y)
					if polyMainGrid[x][y]==0 :
						inside=False
						i=0
						# check if this point is inside a poly found before
						while len(groundPolys)>0 and i < len(groundPolys):
							if pdebug :test.setPixelF(x, y, testin)
							if pointInPoly(x,y,groundPolys[i]) :
								if pdebug :print "is in."
								# we're inside
								while polyMainGrid[x][y]==0 and x<xlast:
									polyMainGrid[x][y]=pxdone
									if pdebug :
										try:test.setPixelF(x, y, fill)
										except:pass
									x +=1
								#if polyMainGrid[x][y]==1 :
								#	if pdebug :test.setPixelF(x, y, blue)
								#	pxpoly=1
								#else :
								inside=True
							if inside : i=len(groundPolys)
							else : i+=1
						if inside==False :
							# we have a new poly there. init
							if pdebug :
								print "____________________"
								print "n%s : "%(len(groundPolys))
							x = x-1
							y = y-1
							xo=x
							yo=y
							#print "start %s,%s" %(x,y)
							#vec=[0,-1,swh]
							dr=0
							dr2=0
							vec=[1,1]
							l=sqrt(2)
							x +=vec[0]
							y +=vec[1]
							poly=[]
							count=0
							if pdebug :
								try:test.setPixelF(xo, yo, verystartcol)
								except:pass
								print "start @ %s %s"%(xo,yo)
							while xo<>x or yo<>y and count<4 :
								#Window.RedrawAll()
								vec2=vec
								#dr2=dr
								# border control : change dir
								while x<xfirst or y<yfirst or x>=xlast or y>=ylast :
									if pdebug : 
										try:test.setPixelF(x, y, notok)
										except : pass
									x -=vec2[0]
									y -=vec2[1]
									l,d=readVec(vec2)
									#if l==1 : l=sqrt(2)
									#else : l=1
									vec2=writeVec(l,d+90)
									vec2=[int(vec2[0]),int(vec2[1])]
									#dr2=(dr2+1)%4
									#vec3=dir[dr2]
									#print "2396",vec2,vec3
									x +=vec2[0]
									y +=vec2[1]
								#if vec<>vec2 and (vec==[1,0] or vec==[0,1]) : poly.append(vec)
								vec=vec2
								if polyMainGrid[x][y]==1 : #pxpoly :
									count=0
									poly.append(vec)
									
									if pdebug :
										print "%s %s ok - dir %s"%(x,y,vec)
										try:test.setPixelF(x, y, current)
										except:pass
									l,d=readVec(vec)
									#if l==1 : l=sqrt(2)
									#else : l=1
									vec2=writeVec(l,d+90)
									vec2=[int(vec2[0]),int(vec2[1])]
									x2=x+vec2[0]
									y2=y+vec2[1]
									#print vec2,l,d
									# border control : change dir
									while x2<xfirst or y2<yfirst or x2>=xlast or y2>=ylast :
										x2 -=vec2[0]
										y2 -=vec2[1]
										l,d=readVec(vec2)
										vec2=writeVec(l,d+90)
										vec2=[int(vec2[0]),int(vec2[1])]
										#print "border",vec2
										x2 +=vec2[0]
										y2 +=vec2[1]
				
									if polyMainGrid[x2][y2]==1 : #pxpoly :
											poly.append(vec2)
											l,d=readVec(vec2)
											#if l<>1 :
											vec2=writeVec(l,d+90)
											vec2=[int(vec2[0]),int(vec2[1])]
											if pdebug : 
												try :test.setPixelF(x2, y2, current)
												except:pass
											x=x2
											y=y2
											vec=vec2
											
											if pdebug :print "%s %s ok - new dir +90 %s"%(x2,y2,vec)

									if xo<>x or yo<>y :
										x +=vec[0]
										y +=vec[1]
								# else change orientation : new vector v > ^ <
								else :
									#poly.append(vec)
									if pdebug :print "%s %s ko"%(x,y)
									x -=vec[0]
									y -=vec[1]
									l,d=readVec(vec)
									if l==1 : l=sqrt(2)
									else : l=1
									vec=writeVec(l,d-45)
									vec=[int(vec[0]),int(vec[1]),0]
									#dr=(dr+1)%4
									#vec3=dir[dr]
									#print "2423",vec2,vec3
									x +=vec[0]
									y +=vec[1]
									
									count +=1
							if pdebug : 
								try :test.setPixelF(xo, yo, finishcol)
								except:pass
							pxpoly=0
							#bpy.data.images.active = test
							if pdebug : Window.RedrawAll()
							polyMainGrid[xo][yo]=1
							x=xo
							y=yo
							#dprint("mark as done..",level=4)
							#for c in poly :
							#	x +=c[0]
							#	y +=c[1]
								#if x<xlast and y<ylast : polyMainGrid[x][y]=pxdone
							# TODO !!
							poly=polyInput(poly,[xo,yo,0])
							poly=polyClean(poly)

							poly=vecToCoord(poly)
							if pdebug :print "final : %s"%(poly)
							groundPolys.append(poly)
							#print groundList
							x=xfirst
							#if y>yfirst : y -=1
							#print x,y
					#last=y
					x +=1
			if pdebug :Window.RedrawAll()
		

			# conformize
			#groundList=polyInput(groundList)
			# resize trick for roads
			groundPolys=polyIn(groundPolys,0.5,type="coord")
			# smooth corners
			if swHsmooth>0 :
				groundPolys=polyCorners(groundPolys,swHbvl,swHsmooth,type="coord")
				smooth=True
			else :
				smooth=False
			# convert to vectors
			
			groundPolys=coordToVec(groundPolys)
			#groundPolys=polyClean(groundPolys)
			for i,poly in enumerate(groundPolys) :
				#print
				#print
				#print
				#print "-----------------------------------------------"
				#print "poly city %s"%(i)
				#print poly
				#poly=polyCorners(poly,swHbvl,swHsmooth,type="coord")
				#poly=coordToVec(poly)
				#print "poly city smooth %s"%(i)
				#print poly
				# locate/resize trick
				poly[0]=[poly[0][0]+mainColsOff,poly[0][1]+mainLinesOff,0]
				createPoly(groundList,poly,[2,3],shape=swbevel,smooth=smooth,fill=True)
			dprint("main Ground (%s polygons) created in %.2f seconds",(len(groundPolys),sys.time()-timeGround))
	#print "city map dim : %s x %s (%s meters square)" %(mainCols,mainLines,mainCols*citySZ)
	#print "maxCol: %s MinCol: %s" %(maxCol,minCol)
	
				
	if buildingBuild :
		timeBuilding=sys.time()
		dprint("Creating building geometry...")
		totB=len(buildingList)
		B=0
		pBar=0.5
		# orientation - collect informations about objects around each building geometry
		for i,building in enumerate(buildingList) :
			Window.DrawProgressBar(pBar+(i*pBarCeil)/totB,"Building libs working...")
			#print building
			B +=1
			#dprint("%s / %s",(B,buildingList),level=4)
			pos=building[0]
			sz=building[1]
			ffloor=building[2]
			floor=building[3]
			lod=building[4]
			orientation=building[5]
			#([pos,[bsz[0],bsz[1],bsz[2] ],ffloor,floor,lod,orientation])
			around=[["","",""],["","",""],["","",""],["","",""]]
			intPosBsz=[int(pos[0]+bsz[0])+crumb(pos[0]+bsz[0]),int(pos[1]+bsz[1])+crumb(pos[1]+bsz[1])]
			intPos=[int(pos[0]),int(pos[1])]
			# south
			yfill=1
			while intPos[1]-yfill >= 0 and intPos[1]-yfill >= intPos[1]-3 :
				xstart=yfill
				if intPos[0]-xstart < 0 : xstart=0
				if intPosBsz[0]+xstart > mainCols : xend=mainCols
				else : xend=intPosBsz[0]+xstart
				for xfill in range(intPos[0]-xstart,xend) :
					#print xfill,yfill,intPos[0],intPosBsz[0]
					#print mainGrid[xfill][intPosBsz[0]+yfill]
					around[0][yfill-1]=around[0][yfill-1]+str(mainGrid[xfill][intPos[1]-yfill])
				yfill +=1
			# north	
			yfill=1
			while intPosBsz[1]+yfill < mainLines and intPosBsz[1]+yfill <= intPosBsz[1]+3 :
				xstart=yfill
				if intPos[0]-xstart < 0 : xstart=0
				if intPosBsz[0]+xstart > mainCols : xend=mainCols
				else : xend=intPosBsz[0]+xstart
				for xfill in range(intPos[0]-xstart,xend) :
					#print xfill,yfill,intPos[0],intPosBsz[1]
					#0 1 1 5
					#print mainGrid[xfill][intPosBsz[0]+yfill]
					around[2][yfill-1]=around[2][yfill-1]+str(mainGrid[xfill][intPosBsz[1]+yfill])
				yfill +=1
			# west
			xfill=1
			while intPos[0]-xfill >=0 and intPos[0]-xfill >= intPos[0]-3 :
				ystart=xfill
				if intPos[1]-ystart < 0 : ystart=0
				if intPosBsz[1]+ystart > mainLines : yend=mainLines
				else : yend=intPosBsz[1]+ystart
				for yfill in range(intPos[1]-ystart,yend) :
					around[3][xfill-1]=around[3][xfill-1]+str(mainGrid[intPos[0]-xfill][yfill])
				xfill +=1
			# east
			xfill=1
			while intPosBsz[0]+xfill < mainCols and intPosBsz[0]+xfill <= intPosBsz[0]+3 :
				ystart=xfill
				if intPos[1]-ystart < 0 : ystart=0
				if intPosBsz[1]+ystart > mainLines : yend=mainLines
				else : yend=intPosBsz[1]+ystart
				for yfill in range(intPos[1]-ystart,yend) :
					around[1][xfill-1]=around[1][xfill-1]+str(mainGrid[intPosBsz[0]+xfill][yfill])
				xfill +=1			

			"""
			ACATS
			[[1.0, 10, 20, 10, 20], ['building_skycrapers_littleneo', 0.5], ['building_skycrapers_piiichan', 0.5]]
			[[1.0, 10, 20, 10, 20], ['building_residential_littleneo', 0.5], ['building_residential_simple_cube', 0.5]]
			[[1.0, 10, 20, 10, 20], ['building_test2_piiichan', 0.5]]
			[[], ['building_default_littleneo', 0.5], ['building_default_simple_cube', 0.5]]
			"""
			# find all eligible categories for that building
			cats=[]
			tot=0
			val=0
			for i,c in enumerate(ACATS) :
				if i<len(ACATS)-1:
					nbf=((sz[2]-ffloor)/floor)+1
					#print ((sz[2]-ffloor)/floor)+1,c[0][1],c[0][2]
					if nbf>=c[0][1] and nbf<=c[0][2] :
						if min(sz)>=c[0][3] and max(sz)<=c[0][4] :
							cats.append(i)
			# several cat possible : random according to each cat chances
			if len(cats)>1 :
				for i in cats :tot +=ACATS[i][0][0]
				random=Puniform(0,tot)
				for i in cats :
					if random<=val+ACATS[i][0][0] : 
						cat=i
						break
					else : val +=ACATS[i][0][0]
			# no cat : use default
			elif len(cats)==0 :
				cat=-1
			# one cat only : just test if chances are 1, else random between default and cat
			else :
				cat=cats[0]
				if ACATS[cats[0]][0][0]<1 :
					defchances=1-ACATS[cats[0]][0][0]
					random=Puniform(0,1)
					if random>=ACATS[cats[0]][0][0] :cat=-1
			
			# choose library inside the chosen category cat
			tot=0
			val=0
			for i in range(1,len(ACATS[cat])) :tot +=ACATS[cat][i][1]
			random=Puniform(0,tot)
			for i in range(1,len(ACATS[cat])) :
				if random<=val+ACATS[cat][i][1] :
					funcname=ACATS[cat][i][0]
					break
				else : val +=ACATS[cat][i][1]

			#funcargs="([vs,fs,sm_pop,mat_pop],[pos[0]+mainColsOff,pos[1]+mainLinesOff,pos[2]],sz,ffloor,floor,lod,around)"
			funcargs="(cityList,[pos[0]+mainColsOff,pos[1]+mainLinesOff,pos[2]],sz,ffloor,floor,lod,around)"
			eval(funcname+funcargs)
			
		dprint("%s Buildings created in %.2f seconds",(len(buildingList),sys.time()-timeBuilding))
						#array.append([pos,bsz,lod,float])
						#if streetN > 0 : # +streetS+streetE+streetN
						#	pos[2]=0
						#	bsz[2]=1
						# Give building main properties to chosen building library
						#pastpseudoInc=pseudoInc
						#createBuilding(vs,fs,sm_pop,mat_pop,uv,(pos[0]+mainColsOff,pos[1]+mainLinesOff,pos[2]),bsz,ffloor,floor,lod,orientation)
						#if failed<>"" : failed +="library : %s" %(libPath)
						#pseudoInc=pastpseudoInc
						#if expandE+expandN > 0 or spawnIt==1 :
						#	createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-30),(aX-expandE,aY-expandN,2),1,1,lod,orientation)
						#	createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-20),(aX,aY,2),1,lod,orientation)
						#	createBuilding(vs,fs,sm_pop,mat_pop,uv,(x+mainColsOff,y+mainLinesOff,-100),(aX,aY,10),1,lod,orientation)
		
	#return cityList, [vs_grd, fs_grd, mat_grd,sm_grd], failed
	return cityList, groundList, failed

def createCity(pms,rpm):
	global buildingBuild,groundBuild,popMapSourceType,groundMapSourceType,groundInfluence,buildingInfluence

	# PARAMETERS:
	# pms: population map size
	# rpm: randomize population map
	# hf: height factor
	# hdf: height difference factor
	# rlf: random location factor
	# bSizeInt[0]: minimum building size
	# bSizeInt[1]: maximum building size
	global timeStart,failed,pBar
	global pseudoInc
	global popMap,groundMap
	pseudoInc=0
	lod=SMDB[0].val
	timeStart = sys.time()
	pBar=0
	Window.DrawProgressBar(pBar,"init...")
	print "Building city......"
	sc=Scene.GetCurrent()
	texs=[0,0]
	texoffst=[0,0]

	#"Source %t|Image (diteur UV) %x0|Image %x1|Material %x3"
	if popMapSourceType==0 :popMapSource=imageUVlist[MPRM[0].val]
	elif popMapSourceType==1 : popMapSource=TLIMB[0].val
	elif popMapSourceType==3 : popMapSource="maps_"+mapMatList[MPMM[0].val]
	popMapRnd=[TRPM[0].val,TRPS[0].val]
	
	if groundMapSourceType==0 :groundMapSource=imageUVlist[MGRM[0].val]
	elif groundMapSourceType==1 : groundMapSource=TLIM[0].val
	elif groundMapSourceType==3 : groundMapSource="maps_"+mapMatList[MGMM[0].val]	
	groundMapRnd=[TRGM[0].val,TRGS[0].val]

	# save current settings that we'll have to change during the execution
	render=sc.getRenderingContext()
	prevImageType=render.imageType
	prevCompositor=render.compositor
	prevAspectX=render.aspectX
	prevAspectY=render.aspectY
	prevSizeX=render.sizeX
	prevSizeY=render.sizeY
	prevCamera=sc.objects.camera
	prevLayers=sc.layers
	prevRenderwinSize=render.renderwinSize
	prevImageType=render.imageType
	prevOSA=render.oversampling
	prevRenderPath=render.getRenderPath()
	editmode = Window.EditMode()
	if editmode: Window.EditMode(0)	

	
	# STUFF CREATION/UPDATES
	
	# CITY mesh & object
	try:me=Mesh.Get("city")
	except NameError:me=Mesh.New("city")
	try:ob=Object.Get("city")
	except ValueError:
		ob=Object.New("Mesh", "city")
		ob.link(me)
		sc.objects.link(ob)
	# GROUND mesh & object
	try:grdMe=Mesh.Get("cityground")
	except NameError:grdMe=Mesh.New("cityground")
	try:grdOb=Object.Get("cityground")
	except ValueError:
		grdOb=Object.New("Mesh", "cityground")
		grdOb.link(grdMe)
		sc.objects.link(grdOb)
	# TEMPORARY MESHES - used by libs for building process
	tmpMesh=Mesh.New("tmpBuilding")
	tmpMesh=Mesh.New("tmpMesh")
	try :tmpPoly=Mesh.Get("tmpPoly")
	except NameError:tmpPoly=Mesh.New("tmpPoly")
	try:	obtmp=Object.Get("tmpPoly")
	except:	obtmp=Object.New("Mesh", "tmpPoly")
	obtmp.link(tmpPoly)
	try:sc.objects.link(obtmp)
	except:pass	
	
	# MAP SOURCES
	
	# create/relink the maps
	imagePath=path+"tmp"+sys.sep
	try :
		popMap=Image.Get("popMap")
		popMap.setFilename(imagePath+"popMap.png")
		popMapExist=True
	except :
		print "popmap failed"
		popMapExist=False
	try :
		groundMap=Image.Get("groundMap")
		groundMap.setFilename(imagePath+"groundMap.png")
		groundMapExist=True
	except :
		print "groundmap failed"
		groundMapExist=False
		
	# CAMERA for material sources
	camDatName="popCamDat"
	try:camDat=Camera.Get(camDatName)
	except NameError:camDat=Camera.New("ortho",camDatName)
	camDat.scale=2
	camObName="popCamOb"
	try:camOb=Object.Get(camObName)
	except ValueError:camOb=sc.objects.new(camDat,camObName)
	sc.objects.camera=camOb
	camOb.loc=[0,0,-49]
	camOb.rot=[0,0,0]
	camOb.layers=[20]
	
	# create default textures,materials, mesh, objects for map as material sources 
	# popmap :
	try:popMapTex=Texture.Get("popMapTex")
	except NameError:
		popMapTex=Texture.New("popMapTex")
		popMapTex.setType("Stucci")
	try:popMapMat=Material.Get("maps_building_default")
	except NameError:
		popMapMat=Material.New("maps_building_default")
		popMapMat.mode|=Material.Modes.SHADELESS
		popMapMat.setTexture(0, popMapTex)
		popMapMat.rgbCol=[0,0,0]
		tex=popMapMat.getTextures()[0]
		tex.col=(1,1,1)
	#mesh & object - default  material
	popMapMeName="popMapMe"
	popMapObName="popMapOb"
	try:popMapMe=Mesh.Get(popMapMeName)
	except NameError:
		popMapMe=Mesh.Primitives.Plane()
		popMapMe.name=popMapMeName
	try:popMapOb=Object.Get(popMapObName)
	except ValueError:
		popMapOb=sc.objects.new(popMapMe,popMapObName)
		popMapObData=popMapOb.getData()
		popMapObData.materials = [popMapMat]
		popMapObData.update()
		Blender.Window.RedrawAll()
		
	if buildingBuild and popMapSourceType==3 :
		popMapMat=Material.Get(popMapSource)
		texs[0]=popMapMat.getTextures()
		# randomize map texture offset
		if popMapRnd[0]:
			if popMapRnd[1]:
				texoffst[0]=[Puniform(-10,10),Puniform(-10,10),Puniform(-10,10)]
			else : texoffst[0]=[uniform(-10,10),uniform(-10,10),uniform(-10,10)]
			for t in texs[0] :
				if str(type(t))=="<type 'Blender MTex'>" :
					t.ofs=(
					((t.ofs[0]+texoffst[0][0])%20)-10,
					((t.ofs[1]+texoffst[0][1])%20)-10,
					((t.ofs[2]+texoffst[0][2])%20)-10
					)
				
	if groundInfluence and groundMapSourceType==3 :
		groundMapMat=Material.Get(groundMapSource)
		texs[1]=groundMapMat.getTextures()
		# randomize map texture offset
		if groundMapRnd[0]:
			if groundMapRnd[1]:
				texoffst[1]=[Puniform(-10,10),Puniform(-10,10),Puniform(-10,10)]
			else : texoffst[1]=[uniform(-10,10),uniform(-10,10),uniform(-10,10)]
			for t in texs[1] :
				#print t,type(t)
				if str(type(t))=="<type 'Blender MTex'>" :
					
					t.ofs=(
					((t.ofs[0]+texoffst[1][0])%20)-10,
					((t.ofs[1]+texoffst[1][1])%20)-10,
					((t.ofs[2]+texoffst[1][2])%20)-10
					)
		
	popMapOb.loc=[0,0,-50]
	popMapOb.layers=[20]
	


	ob.layers=prevLayers	
	grdOb.layers=prevLayers	
	
	# POPMAP SOURCE
	#if TMBG[0].val :
		# copy selected image from UVeditor to popmap
	if popMapSourceType==0 :
		popMapSource=Image.Get(popMapSource)
		UVimagePath=popMapSource.getFilename()
		popMapSource.setFilename(imagePath+"popMap.png")
		popMapSource.save()
		popMapSource.setFilename(UVimagePath) # restore image path
		#except : pass # against packed image warning
	# copy image from file system to popmap
	if popMapSourceType==1 :
		shutil.copy(path+popMapSource, imagePath+"popMap.png")
	# render popMap
	if popMapSourceType==3 :
		popMapObData=popMapOb.getData()
		popMapObData.materials = [popMapMat]
		popMapObData.update()
		Blender.Window.RedrawAll()
		sc.layers=[20]
		render.aspectX=render.aspectY=1
		render.sizeX=SPMX[0].val
		render.sizeY=SPMY[0].val
		render.renderwinSize=100
		render.compositor=False
		render.oversampling=False
		render.imageType=Scene.Render.PNG
		
		render.displayMode=1
		render.render()
		render.setRenderPath(imagePath)
		render.saveRenderedImage("popMap")
	#if popMapSourceType>=3 :
	if popMapExist :popMap.reload()
	else :
		popMap=Image.Load(imagePath+"popMap.png")
		popMap.setName("popMap")
	bpy.data.images.active=popMap

	# GROUND SOURCE
	#if TMGD[0].val and groundMapSourceType<>2 :
	if groundMapSourceType<>2 :
		# copy selected image from UVeditor to popmap
		if groundMapSourceType==0 :
			groundMapSource=Image.Get(groundMapSource)
			UVimagePath=groundMapSource.getFilename()
			groundMapSource.setFilename(imagePath+"groundMap.png")
			groundMapSource.save()
			groundMapSource.setFilename(UVimagePath) # restore image path
			#except : pass # against packed image warning
		# copy image from file system to popmap
		if groundMapSourceType==1 :
			shutil.copy(path+groundMapSource, imagePath+"groundMap.png")
		# render groundMap
		if groundMapSourceType==3 :
			popMapObData=popMapOb.getData()
			popMapObData.materials = [groundMapMat]
			popMapObData.update()
			Blender.Window.RedrawAll()
			sc.layers=[20]
			render.aspectX=render.aspectY=1
			render.sizeX=SGDX[0].val
			render.sizeY=SGDY[0].val
			render.renderwinSize=100
			render.compositor=False
			render.oversampling=False
			render.imageType=Scene.Render.PNG
			
			render.displayMode=1
			render.render()
			render.setRenderPath(imagePath)
			render.saveRenderedImage("groundMap")

		if groundMapExist :groundMap.reload()
		else :
			groundMap=Image.Load(imagePath+"groundMap.png")
			groundMap.setName("groundMap")
		bpy.data.images.active=groundMap

	Scene.Render.CloseRenderWindow()
	
	# restore all user settings that have been changed during execution
	render.setRenderPath(prevRenderPath)
	render.compositor=prevCompositor
	render.imageType=prevImageType
	render.aspectX=prevAspectX
	render.aspectY=prevAspectY
	render.sizeX=prevSizeX
	render.sizeY=prevSizeY
	sc.objects.camera=prevCamera
	sc.layers=prevLayers
	render.renderwinSize=prevRenderwinSize
	render.imageType=prevImageType
	if prevOSA:render.enableOversampling(1)
	# restore user texture offset
	if  buildingBuild and popMapSourceType==3 and popMapRnd[0] :
		for i,t in enumerate(texs[0]) :
			if str(type(t))=="<type 'Blender MTex'>" :
				t.ofs=(
				((t.ofs[0]-texoffst[0][0])%20)-10,
				((t.ofs[1]-texoffst[0][1])%20)-10,
				((t.ofs[2]-texoffst[0][2])%20)-10
				)
	if groundBuild and groundMapSourceType==3 and groundMapRnd[0] :
		for i,t in enumerate(texs[1]) :
			if str(type(t))=="<type 'Blender MTex'>" :
				t.ofs=(
				((t.ofs[0]-texoffst[1][0])%20)-10,
				((t.ofs[1]-texoffst[1][1])%20)-10,
				((t.ofs[2]-texoffst[1][2])%20)-10
				)
	
	# create the city
	cityMesh, groundMesh, failed=createCityMesh()

	if failed=="" :
		timeUpdate=sys.time()
		Window.DrawProgressBar(0.3,"update Buildings...")
		if groundMapSourceType==2:
			grdMe.verts.delete(range(len(grdMe.verts))) # clear all previous vertices and faces
		if buildingBuild :
			dprint("buildings : %s vertices, %s faces",(len(cityMesh[0]),len(cityMesh[1])),level=4)
			me.verts.delete(range(len(me.verts))) # clear all previous vertices and faces
			me.verts.extend(cityMesh[0])
			me.faces.extend(cityMesh[1])
			# smooth
			for i, f in enumerate(me.faces):f.smooth=cityMesh[2][i]
			# materials
			try :
				for i in range(len(cityMesh[1])) :me.faces[i].mat=cityMesh[3][i]
			except :
				dprint("Library Materials count error. You must link one material per face. Check the used libraries.")
				i=len(cityMesh[1])
		if groundBuild :
			dprint("ground : %s vertices, %s faces",(len(groundMesh[0]),len(groundMesh[1])),level=4)
			Window.DrawProgressBar(0.4,"update Ground...")
			dprint("updated object meshes in %.2f seconds",( sys.time()-timeUpdate))
			grdMe.verts.delete(range(len(grdMe.verts))) # clear all previous vertices and faces
			grdMe.verts.extend(groundMesh[0])
			grdMe.faces.extend(groundMesh[1])
			timeMat=sys.time()
			Window.DrawProgressBar(0.6,"update Ground Materials...")
			# smooth
			for i, f in enumerate(grdMe.faces):f.smooth=groundMesh[2][i]		
			Window.DrawProgressBar(0.8,"clean Mesh...")
			# materials
			try :
				for i in range(len(grdMe.faces)) :grdMe.faces[i].mat=groundMesh[3][i]
			except :
				dprint("Library Materials count error. You must link one material per face. Check the used libraries.")
				i=len(groundMesh[1])
			dprint("Ground materials in %.2f seconds",( sys.time()-timeMat))
			timeClean=sys.time()

			#grdMe.remDoubles(0.0)
			dprint("Ground mesh cleaned in %.2f seconds",( sys.time()-timeClean))
		Window.DrawProgressBar(0.9,"update and restore...")
		Blender.Redraw()

		Window.DrawProgressBar(1.0,"Done.")
		dprint("....... city built.")
		dprint("total time %.2f seconds",(sys.time()-timeStart))
		dprint("<END>")
	else :
		print failed
	#memory, objects, meshs etc cleaning
	vs=fs=mat=vs_grd=fs_grd=mat_grd=[]
	#sc.objects.unlink(obtmp)

	if editmode: Window.EditMode(1)

	

# INIT
# ######################################################################	
# GUI
D,DPB,DT,DN,DM,DS,DST=Draw.Create,Draw.PushButton,Draw.Toggle,Draw.Number,Draw.Menu,Draw.Slider,Draw.String # aliases (Draw Module)
no=111

escTrick=0
# GUI vars
# Main tab
cb=[0,0] 				# button creation button
TMn=[D(True),1]			# toggle tab MAIN
TB=[D(False),2]			# toggle tab BUILDING
TG=[D(False),3]			# toggle tab GROUND
TMp=[D(False),4]		# toggle tab MAPS
BPSV=[0,31]	# save preset
BPSA=[0,32]	# save as preset
BPXX=[0,33]	# delete preset
BPSR=[0,30] # refresh/restore presets
BCSV=[0,37] # config save
BCLD=[0,38] # config load

# Building
BLBR=[0,20]					# button refresh libs
BRFS=[0,19]					# button refresh image list
BCTR=[0,47]					# button refresh categories
TBDM=[D(True),24]			# toggle buildings properties
TBLD=[D(False),25]			# toggle buildings library dispatcher
TBLC=[D(False),23]			# toggle building library configuration

#SRBL=[D(0.0),no] 				# slider random building location
# Ground
SWMM=[D(100),no]			# sidewalks creation method
SWSR=[D(100),no]			# sidewalks method variation : width x width_var or aligned to grid
SBD=[D(9),no]				# slider Building Average Density Meter/pixel
#debug=5

#Map
BLIM=[0,40]					# load image button for ground (image from filesystem)
BLIMB=[0,42]				# load image button for building (image from filesystem)
BRIM=[0,no]					# refresh image button
BMTR=[0,48]					# button refresh maps mat material

# default set
SWSH=[D(5),no,"","","slider",0,10]		# sidewalks Smooth Corners 
SWSV=[D(3),no,"","","slider",0,10]		# sidewalks Smooth Borders
SWB=[D(.5),no,"","","slider",0.1,3.0]		# sidewalks Borders Width

matList=[[],[]]
debugCore=5
# terran marines say : 'go go go' !
sceVersion="v"+sceVersion
dprint ("",level=5)
dprint ("",level=5)
dprint (" Suicidator City Engine",level=5)
dprint (" "+sceVersion,level=5)
dprint ("                  _            ",level=5,pxart=1)
dprint ("   _    ___       .        __  ",level=5,pxart=1)
dprint ("  _.  __ooo_     _.     __ oo  ",level=5,pxart=1)
dprint ("  ..__OOOooo_    .. ___ OO_oo  ",level=5,pxart=1)
dprint (" _.OOOOOOooo. ___.._ooo_OOOoo__",level=5,pxart=1)
dprint (" ooOOOOOOooo._..oooooOOOOOOoooo",level=5,pxart=1)
dprint (" ooOOOOOOOOo....ooOOOOOOOOOOOoo",level=5,pxart=1)
dprint (" ooOOOOOOOOo    ooOOOOOOOOOOOoo",level=5,pxart=1)
dprint ("",level=5)
dprint (" jerome.le.chat.free.fr/index.php/en/blender/scripts/city-engine.html")
dprint (" for updates and documentation",level=5)
dprint ("",level=5)
refreshConfig=[True,"init"]
Draw.Register(gui,event,bevent)

